from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.tunnel_category import TunnelCategory
from ..models.vehicle_category import VehicleCategory
from ..models.vehicle_engine_type import VehicleEngineType
from ..models.vehicle_type import VehicleType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Vehicle")


@_attrs_define
class Vehicle:
    """Vehicle-specific parameters

    Attributes:
        shipped_hazardous_goods (str | Unset): Hazardous goods restrictions refer to the limitations and regulations
            imposed on the transportation of specific types of hazardous materials during a trip.

            A comma-separated list of hazardous goods being shipped in the vehicle. The following values are possible:

            * `explosive`: Materials that are capable of causing an explosion.
            * `gas`: Gas (definition varies from country to country). For details, check
            [here](https://en.wikipedia.org/wiki/HAZMAT_Class_2_Gases).
            * `flammable`: Materials that are easily ignited and capable of catching fire.
            * `combustible`: Materials that have the potential to burn or catch fire.
            * `organic`: Materials derived from living organisms or containing carbon compounds.
            * `poison`: Substances that can cause harm or death when ingested, inhaled, or absorbed.
            * `radioactive`: Materials that emit radiation and pose potential health risks.
            * `corrosive`: Substances that can cause damage or destruction through chemical reactions.
            * `poisonousInhalation`: Materials that are toxic when inhaled.
            * `harmfulToWater`: Materials that can cause pollution or harm to water bodies.
            * `other`: Other types of hazardous materials not covered by the above categories.

            **Note:** Supported in `truck`, `bus`, `privateBus`, `car` (Beta), `taxi` (Beta) transport modes.
             Example: explosive,gas,flammable.
        gross_weight (int | Unset): Gross vehicle weight, including trailers and shipped goods when loaded at capacity,
            specified in kilograms.

            If unspecified, it will default to currentWeight. If neither parameter has a value specified, it will default to
            0.

            **Notes:**
            * Supported in `truck`, `bus`, `privateBus`, `car` (Beta), `taxi` (Beta) transport modes.
            * Maximum weight for a car or taxi _without_ a trailer is 4250 kg.
            * Maximum weight for a car or taxi _with_ a trailer is 7550 kg.
        current_weight (int | Unset): Current vehicle weight, including trailers and shipped goods currently loaded,
            specified in kilograms.

            If unspecified, it will default to grossWeight. If neither parameter has a value specified, it will default to
            0.

            **Notes:**
            * Supported in `truck`, `bus`, `privateBus`, `car` (Beta), `taxi` (Beta) transport modes.
            * Maximum weight for a car or taxi _without_ a trailer is 5000 kg.
            * Maximum weight for a car or taxi _with_ a trailer is 8500 kg.
            * A route request with `currentWeight` above `grossWeight` may result in non-compliant or invalid routes.
        empty_weight (int | Unset): Empty weight of the vehicle combination ready to be driven, including necessary
            fluids and equipment
            determined by the local regulations, specified in kilograms.

            **Note:**
            * Supported in `truck`, `bus`, `privateBus`, `car` (Beta), `taxi` (Beta) transport modes.
            * Maximum weight for a car or taxi _without_ a trailer is 4250 kg.
            * Maximum weight for a car or taxi _with_ a trailer is 7550 kg.
        weight_per_axle (int | Unset): Heaviest vehicle weight-per-axle, specified in kilograms.

            Heaviest weight-per-axle, regardless of axle-type or axle-group. It is evaluated against
            all axle-weight restrictions, including single-axle and tandem-axle weight restrictions.
            It is useful if differences between axle types, like tandem and triple axles, are not
            relevant. This is the case in many countries, since they don't distinguish between these
            different axle groups on signs and in regulations.

            More fine-grained axle weight input is possible with `weightPerAxleGroup`.

            **Note:** `weightPerAxleGroup` and `weightPerAxle` are incompatible.

            **Note:** Supported in `truck`, `bus`, `privateBus`, `car` (Beta), `taxi` (Beta) transport modes.
        weight_per_axle_group (str | Unset): Specifies the weights of different axle groups, such as single and tandem
            axles.

            This allows specification of axle weights in a more fine-grained way than `weightPerAxle`. This
            is relevant in countries with signs and regulations that specify different limits for different
            axle groups, such as the USA and Sweden.

            All axle group weights are evaluated against their respective axle group restrictions and against
            generic axle weight restrictions. This means that the provided tandem axle group weight is
            compared with all tandem axle group weight restrictions and all generic axle weight restrictions.
            The same is true for single, triple, quad, and quint axle groups.

            Format: `AxleGroup:Weight[,AxleGroup2:Weight2]...`

            Currently, allowed axle-groups are:
              * `single`
              * `tandem`
              * `triple`
              * `quad`
              * `quint`

            Weights are specified in kilograms (kg) and represent the total weight of the axle-group.

            **Note:** `weightPerAxleGroup` and `weightPerAxle` are incompatible.

            **Limitations:** only valid when `transportMode` is one of (`truck`, `bus`, `privateBus`).
             Example: single:11000,tandem:18000.
        height (int | Unset): Vehicle height, specified in centimeters.

            **Note:** Supported in `truck`, `bus`, `privateBus`, `car` (Beta), `taxi` (Beta) transport modes.
        width (int | Unset): Vehicle width, specified in centimeters.

            **Note:** Supported in `truck`, `bus`, `privateBus`, `car` (Beta), `taxi` (Beta) transport modes.
        length (int | Unset): Vehicle length, specified in centimeters.

            **Note:** Supported in `truck`, `bus`, `privateBus`, `car` (Beta), `taxi` (Beta) transport modes.
        kpra_length (int | Unset): Kingpin to rear axle length, in centimeters.

            **NOTE:** Currently, the KPRA restrictions are only present in California and Idaho.

            **Note:** Supported in `truck`, `car` (Beta), `taxi` (Beta) transport modes.
        payload_capacity (int | Unset): Allowed payload capacity, including trailers, specified in kilograms.

            **Note:** Supported in `truck`, `car` (Beta), `taxi` (Beta) transport modes.
        tunnel_category (TunnelCategory | Unset): Specifies the tunnel category used to restrict the transport of
            specific goods.
        axle_count (int | Unset): Specifies the total number of axles the vehicle has, i.e., axles on the base vehicle
            and any attached trailers.

            **Note:** Supported in `truck`, `bus`, `privateBus`, `car` (Beta), `taxi` (Beta) transport modes.
        tires_count (int | Unset): Specifies the total number of tires the vehicle has, i.e., the tires on the base
            vehicle and any attached trailers.
        category (VehicleCategory | Unset): Specifies the category of the vehicle. The supported values are:

            * `undefined`: The vehicle category is undefined, and no special considerations are taken into
              account. Vehicle routing will proceed as normal.
            * `lightTruck`: The vehicle is a truck light enough to be classified more as a car than as a truck.
              This exempts it from many legal restrictions for normal trucks. However, restrictions related to the physical
            dimensions of the truck or its cargo still apply.

              For more details, refer to [Truck categories](https://www.here.com/docs/bundle/routing-api-developer-
            guide-v8/page/concepts/truck-routing.html#truck-categories).

              **Note:** Supported only in `truck` transport mode.
             Default: VehicleCategory.UNDEFINED.
        trailer_count (int | Unset): The number of trailers attached to the vehicle.

            Maximum value when used with `transportMode=car` or `transportMode=taxi` is 1.

            **Limitations:** Considered for route calculation when `transportMode` is one of (`truck`, `bus`, `privateBus`).
            Considered for route calculation for restrictions, but not for speed limits, when `transportMode` is `car` or
            `taxi`.
             Default: 0.
        hov_occupancy (int | Unset): **Note**: This parameter is deprecated, `vehicle[occupancy]` and `allow[hov]`
            should be used instead.
            The number of occupants in the vehicle, defined as individuals occupying a seat with a restraint device.
            This value affects the ability of the router to use HOV (High-Occupancy Vehicles) restricted lanes.

            Limitations:
              * Any value over 1 is interpreted as the ability to use any HOV lane, including those restricted to 3+
            passengers.
             Default: 1.
        license_plate (str | Unset): Specifies the information about the vehicle's license plate number.
            This information is used to evaluate whether certain vehicle restrictions in environmental zones apply.
            Currently, only the last character of the license plate can be provided.

            Format: `lastCharacter:{character}`

            Example: `lastCharacter:2`
        speed_cap (float | Unset): Specifies the maximum speed, in meters per second (m/s), that the user wishes not to
            exceed.
            This parameter affects the route's estimated time of arrival (ETA) and consumption calculation.

            Limitations:
              * valid for following transport modes: `car`, `truck`, `scooter`, `taxi`, `bus`, and `privateBus`

            Notes:
              * Car and Truck mode updates route ETA.
              * Scooter mode updates route optimization and ETA.
              * Resulting ETA can not be decreased by this parameter.
        speed_cap_per_fc (str | Unset): A comma separated list of speeds in meters per second (m/s) specifying the
            maximum speed per FC (Functional Class, as defined in the `FunctionalClass` schema) that the user wishes not to
            exceed.
            The list can have up to 5 entries, corresponding to FC 1 to FC 5. Empty entries indicate that no speed cap is
            specified for the corresponding FC.
            This parameter affects the route's estimated time of arrival (ETA) and consumption calculation.

            Limitations:
              * valid for following transport modes: `car`, `truck`

            Values limitations:
              * minimum: 1.00
              * maximum: 70.00

            Examples:
              * `vehicle[speedCapPerFc]=65,50.5,40,30,10` - specify limits for all FCs
              * `vehicle[speedCapPerFc]=65,50.5,,30,10` - specify limits for all FCs except for FC 3
              * `vehicle[speedCapPerFc]=65,50.5,40` - specify limits for FC 1, 2 and 3

            Notes:
              * Resulting ETA can not be decreased by this parameter.
              * vehicle[speedCapPerFc] and vehicle[speedCap] may be used in combination. For FCs where both values are
            specified, the minimum value is used.
        engine_size_cc (int | Unset): Specifies the engine size of the vehicle in cubic centimeters.
            Currently, the value is used only in scooter mode.
            This parameter is utilized to determine if the scooter can be classified as a moped.
            Scooters with an engine size less than 51cc are considered mopeds.
            **Alpha**: This API is in development. It may not be stable and is subject to change. It may have no impact on
            response.
        occupancy (int | Unset): The number of occupants on the vehicle, defined as individuals occupying a seat.
            This value affects the ability of the router to use HOV (High-Occupancy Vehicles) restricted lanes. If not set
            and `allow[hov]=true` is set, then occupancy requirements for all HOV conditions are considered to be met.
            This parameter also affects scooter occupancy restrictions. If not set, occupancy restrictions for scooters are
            ignored.

            The parameter is also used for calculating the toll cost, in cases where the toll-cost for the vehicle depends
            on the passenger count. If not set, occupancy of 1 is assumed for toll calculation.

            **Note:**: This parameter can't be used with 'vehicle[hovOccupancy]'.
        engine_type (VehicleEngineType | Unset): Specifies the engine type of the vehicle.

            **Limitations:**  Currently this parameter is used only for toll calculation. It may be used at any point for
            route calculation (for example, to automatically avoid environmental zones).

            **Note:**
              * If engineType is specified as `internalCombustion`, valid `fuel[type]` should also be provided to get fuel
            specific toll. If `fuel[type]` is not provided, default toll without considering `fuel[type]` will be returned.
              * If `ev` or `fuel` namespace parameters are provided with an incompatible `engineType` (e.g.
            engineType=internalCombustion is provided along with `ev` namespace parameters) then `ev` namespace will
            override `engineType` and incompatible `engineType` will be ignored for toll, consumption, and emission
            computations.
        frontal_area (float | Unset): Frontal area represents the total cross section area of the vehicle as viewed from
            the front, specified in square meters. Physical consumption model is using this value in combination with
            `airDragCoefficient` to calculate the consumption caused by air resistance.
            **Note:** This attribute, or both `width` and `height`, is required when using the physical model
            `consumptionModel=physical` for EV consumption.

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
        rolling_resistance_coefficient (float | Unset): Rolling resistance refers to the resistance experienced by your
            vehicle tire as it rolls over a surface. The main causes of this resistance are tire deformation, wing drag, and
            friction with the ground. The coefficient of rolling resistance is a numerical value indicating the severity of
            this factor.
            **Note:** This attribute is required when using the physical model `consumptionModel=physical` for EV
            consumption.

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
        air_drag_coefficient (float | Unset): The drag coefficient of an vehicle defines the way the vehicle is expected
            to pass through the surrounding air. More streamlined vehicles are more aerodynamic and therefore have smaller
            drag coefficient.
            **Note:** This attribute is required when using the physical model `consumptionModel=physical` for EV
            consumption.

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
        type_ (VehicleType | Unset): Specifies the type of the vehicle

            * `StraightTruck`: A truck on a single frame with a permanently attached cargo area. **Note:**
            default value when truck routing mode is used.
            * `Tractor`: A towing vehicle that can pull one or more semi-trailers (also known as a semi-truck).

            **Limitations:** only valid for `transportMode=truck`.
    """

    shipped_hazardous_goods: str | Unset = UNSET
    gross_weight: int | Unset = UNSET
    current_weight: int | Unset = UNSET
    empty_weight: int | Unset = UNSET
    weight_per_axle: int | Unset = UNSET
    weight_per_axle_group: str | Unset = UNSET
    height: int | Unset = UNSET
    width: int | Unset = UNSET
    length: int | Unset = UNSET
    kpra_length: int | Unset = UNSET
    payload_capacity: int | Unset = UNSET
    tunnel_category: TunnelCategory | Unset = UNSET
    axle_count: int | Unset = UNSET
    tires_count: int | Unset = UNSET
    category: VehicleCategory | Unset = VehicleCategory.UNDEFINED
    trailer_count: int | Unset = 0
    hov_occupancy: int | Unset = 1
    license_plate: str | Unset = UNSET
    speed_cap: float | Unset = UNSET
    speed_cap_per_fc: str | Unset = UNSET
    engine_size_cc: int | Unset = UNSET
    occupancy: int | Unset = UNSET
    engine_type: VehicleEngineType | Unset = UNSET
    frontal_area: float | Unset = UNSET
    rolling_resistance_coefficient: float | Unset = UNSET
    air_drag_coefficient: float | Unset = UNSET
    type_: VehicleType | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        shipped_hazardous_goods = self.shipped_hazardous_goods

        gross_weight = self.gross_weight

        current_weight = self.current_weight

        empty_weight = self.empty_weight

        weight_per_axle = self.weight_per_axle

        weight_per_axle_group = self.weight_per_axle_group

        height = self.height

        width = self.width

        length = self.length

        kpra_length = self.kpra_length

        payload_capacity = self.payload_capacity

        tunnel_category: str | Unset = UNSET
        if not isinstance(self.tunnel_category, Unset):
            tunnel_category = self.tunnel_category.value

        axle_count = self.axle_count

        tires_count = self.tires_count

        category: str | Unset = UNSET
        if not isinstance(self.category, Unset):
            category = self.category.value

        trailer_count = self.trailer_count

        hov_occupancy = self.hov_occupancy

        license_plate = self.license_plate

        speed_cap = self.speed_cap

        speed_cap_per_fc = self.speed_cap_per_fc

        engine_size_cc = self.engine_size_cc

        occupancy = self.occupancy

        engine_type: str | Unset = UNSET
        if not isinstance(self.engine_type, Unset):
            engine_type = self.engine_type.value

        frontal_area = self.frontal_area

        rolling_resistance_coefficient = self.rolling_resistance_coefficient

        air_drag_coefficient = self.air_drag_coefficient

        type_: str | Unset = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if shipped_hazardous_goods is not UNSET:
            field_dict["shippedHazardousGoods"] = shipped_hazardous_goods
        if gross_weight is not UNSET:
            field_dict["grossWeight"] = gross_weight
        if current_weight is not UNSET:
            field_dict["currentWeight"] = current_weight
        if empty_weight is not UNSET:
            field_dict["emptyWeight"] = empty_weight
        if weight_per_axle is not UNSET:
            field_dict["weightPerAxle"] = weight_per_axle
        if weight_per_axle_group is not UNSET:
            field_dict["weightPerAxleGroup"] = weight_per_axle_group
        if height is not UNSET:
            field_dict["height"] = height
        if width is not UNSET:
            field_dict["width"] = width
        if length is not UNSET:
            field_dict["length"] = length
        if kpra_length is not UNSET:
            field_dict["kpraLength"] = kpra_length
        if payload_capacity is not UNSET:
            field_dict["payloadCapacity"] = payload_capacity
        if tunnel_category is not UNSET:
            field_dict["tunnelCategory"] = tunnel_category
        if axle_count is not UNSET:
            field_dict["axleCount"] = axle_count
        if tires_count is not UNSET:
            field_dict["tiresCount"] = tires_count
        if category is not UNSET:
            field_dict["category"] = category
        if trailer_count is not UNSET:
            field_dict["trailerCount"] = trailer_count
        if hov_occupancy is not UNSET:
            field_dict["hovOccupancy"] = hov_occupancy
        if license_plate is not UNSET:
            field_dict["licensePlate"] = license_plate
        if speed_cap is not UNSET:
            field_dict["speedCap"] = speed_cap
        if speed_cap_per_fc is not UNSET:
            field_dict["speedCapPerFc"] = speed_cap_per_fc
        if engine_size_cc is not UNSET:
            field_dict["engineSizeCc"] = engine_size_cc
        if occupancy is not UNSET:
            field_dict["occupancy"] = occupancy
        if engine_type is not UNSET:
            field_dict["engineType"] = engine_type
        if frontal_area is not UNSET:
            field_dict["frontalArea"] = frontal_area
        if rolling_resistance_coefficient is not UNSET:
            field_dict["rollingResistanceCoefficient"] = rolling_resistance_coefficient
        if air_drag_coefficient is not UNSET:
            field_dict["airDragCoefficient"] = air_drag_coefficient
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        shipped_hazardous_goods = d.pop("shippedHazardousGoods", UNSET)

        gross_weight = d.pop("grossWeight", UNSET)

        current_weight = d.pop("currentWeight", UNSET)

        empty_weight = d.pop("emptyWeight", UNSET)

        weight_per_axle = d.pop("weightPerAxle", UNSET)

        weight_per_axle_group = d.pop("weightPerAxleGroup", UNSET)

        height = d.pop("height", UNSET)

        width = d.pop("width", UNSET)

        length = d.pop("length", UNSET)

        kpra_length = d.pop("kpraLength", UNSET)

        payload_capacity = d.pop("payloadCapacity", UNSET)

        _tunnel_category = d.pop("tunnelCategory", UNSET)
        tunnel_category: TunnelCategory | Unset
        if isinstance(_tunnel_category, Unset):
            tunnel_category = UNSET
        else:
            tunnel_category = TunnelCategory(_tunnel_category)

        axle_count = d.pop("axleCount", UNSET)

        tires_count = d.pop("tiresCount", UNSET)

        _category = d.pop("category", UNSET)
        category: VehicleCategory | Unset
        if isinstance(_category, Unset):
            category = UNSET
        else:
            category = VehicleCategory(_category)

        trailer_count = d.pop("trailerCount", UNSET)

        hov_occupancy = d.pop("hovOccupancy", UNSET)

        license_plate = d.pop("licensePlate", UNSET)

        speed_cap = d.pop("speedCap", UNSET)

        speed_cap_per_fc = d.pop("speedCapPerFc", UNSET)

        engine_size_cc = d.pop("engineSizeCc", UNSET)

        occupancy = d.pop("occupancy", UNSET)

        _engine_type = d.pop("engineType", UNSET)
        engine_type: VehicleEngineType | Unset
        if isinstance(_engine_type, Unset):
            engine_type = UNSET
        else:
            engine_type = VehicleEngineType(_engine_type)

        frontal_area = d.pop("frontalArea", UNSET)

        rolling_resistance_coefficient = d.pop("rollingResistanceCoefficient", UNSET)

        air_drag_coefficient = d.pop("airDragCoefficient", UNSET)

        _type_ = d.pop("type", UNSET)
        type_: VehicleType | Unset
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = VehicleType(_type_)

        vehicle = cls(
            shipped_hazardous_goods=shipped_hazardous_goods,
            gross_weight=gross_weight,
            current_weight=current_weight,
            empty_weight=empty_weight,
            weight_per_axle=weight_per_axle,
            weight_per_axle_group=weight_per_axle_group,
            height=height,
            width=width,
            length=length,
            kpra_length=kpra_length,
            payload_capacity=payload_capacity,
            tunnel_category=tunnel_category,
            axle_count=axle_count,
            tires_count=tires_count,
            category=category,
            trailer_count=trailer_count,
            hov_occupancy=hov_occupancy,
            license_plate=license_plate,
            speed_cap=speed_cap,
            speed_cap_per_fc=speed_cap_per_fc,
            engine_size_cc=engine_size_cc,
            occupancy=occupancy,
            engine_type=engine_type,
            frontal_area=frontal_area,
            rolling_resistance_coefficient=rolling_resistance_coefficient,
            air_drag_coefficient=air_drag_coefficient,
            type_=type_,
        )

        vehicle.additional_properties = d
        return vehicle

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.speed_limit_direction import SpeedLimitDirection
from ..models.speed_limit_source import SpeedLimitSource
from ..models.speed_limit_speed_unit import SpeedLimitSpeedUnit
from ..types import UNSET, Unset

T = TypeVar("T", bound="SpeedLimit")


@_attrs_define
class SpeedLimit:
    """
    Attributes:
        max_speed (int): Used in combination with speed limit unit to indicate allowable speed limit at a location, for
            unlimited speed, the value is "999"
        speed_unit (SpeedLimitSpeedUnit): Country defined speed unit
        direction (SpeedLimitDirection | Unset): Rendered in compass value, which is N, S, E, W, NE, SE, NW, SW
        source (SpeedLimitSource | Unset): A generalised identification of the source of `Speed Limit` information.

            Description of supported values:

            - `derived`: Based on administrative regulations, for example: highway exit/entry ramps that do not have posted
            speed limits.
            - `posted`: Based on a posted speed limit sign, speed limit information painted on the road, or data obtained
            from official sources.
    """

    max_speed: int
    speed_unit: SpeedLimitSpeedUnit
    direction: SpeedLimitDirection | Unset = UNSET
    source: SpeedLimitSource | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        max_speed = self.max_speed

        speed_unit = self.speed_unit.value

        direction: str | Unset = UNSET
        if not isinstance(self.direction, Unset):
            direction = self.direction.value

        source: str | Unset = UNSET
        if not isinstance(self.source, Unset):
            source = self.source.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "maxSpeed": max_speed,
            "speedUnit": speed_unit,
        })
        if direction is not UNSET:
            field_dict["direction"] = direction
        if source is not UNSET:
            field_dict["source"] = source

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        max_speed = d.pop("maxSpeed")

        speed_unit = SpeedLimitSpeedUnit(d.pop("speedUnit"))

        _direction = d.pop("direction", UNSET)
        direction: SpeedLimitDirection | Unset
        if isinstance(_direction, Unset):
            direction = UNSET
        else:
            direction = SpeedLimitDirection(_direction)

        _source = d.pop("source", UNSET)
        source: SpeedLimitSource | Unset
        if isinstance(_source, Unset):
            source = UNSET
        else:
            source = SpeedLimitSource(_source)

        speed_limit = cls(
            max_speed=max_speed,
            speed_unit=speed_unit,
            direction=direction,
            source=source,
        )

        speed_limit.additional_properties = d
        return speed_limit

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ChargingStationBrand")


@_attrs_define
class ChargingStationBrand:
    """Information regarding the charging station brand

    Attributes:
        name (str | Unset): Charging station brand name
        hrn (str | Unset): Charging station brand unique ID.
            If specified in `ev[preferredBrands]` parameter
            then it would apply preference to adding stations of the given brand.

            **NOTE:** As of now it is generated as a brand name hash.
    """

    name: str | Unset = UNSET
    hrn: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        hrn = self.hrn

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if hrn is not UNSET:
            field_dict["hrn"] = hrn

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name", UNSET)

        hrn = d.pop("hrn", UNSET)

        charging_station_brand = cls(
            name=name,
            hrn=hrn,
        )

        charging_station_brand.additional_properties = d
        return charging_station_brand

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="BaseEVPhysicalModel")


@_attrs_define
class BaseEVPhysicalModel:
    """Specifies parameters required for calculation of energy consumption for electric vehicles using physical model.

    Attributes:
        drive_efficiency (float): The proportion of the energy drawn from the battery that is used to move the vehicle.
            (This is to factor in energy losses through heat in the motors, for example.)
            **Note:** This attribute is required when using the physical model `consumptionModel=physical` for EV
            consumption.

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
        recuperation_efficiency (float): The proportion of the energy gained when braking or going downhill that can be
            recuperated and restored as battery charge.
            **Note:** This attribute is required when using the physical model `consumptionModel=physical` for EV
            consumption.

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
        auxiliary_power_consumption (float | Unset): Power (in W) consumed by the vehicle's auxiliary systems (for
            example, air conditioning, lights).

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
        air_density (float | Unset): The density of air (in kg/m³) used for calculating aerodynamic drag.

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
             Default: 1.225.
        auxiliary_power_curve (str | Unset): Function curve specifying power consumed by the vehicle's auxiliary systems
            as a function of time.
            Use this instead of the single-valued `auxiliaryPowerConsumption` when auxiliary power consumption is not
            constant.

            The format of the string is a pipe separated list of number pairs, as follows:

            ```
            <TIME_0>,<POWER_0>|<TIME_1>,<POWER_1>|...|<TIME_N>,<POWER_N>
            ```

            where time values are strictly increasing non-negative integer values in seconds, representing elapsed driving
            time since the start of the route, and power values are non-negative floating point values in watts.
            The function is piecewise constant, e.g., for any elapsed time in the range `[TIME_0, TIME_1]` the value of the
            function is `POWER_0`.
            For time greater than `TIME_N`, the value of the function is `POWER_N`. TIME_0 must be explicitly set to zero.
            The time in the last pair, `TIME_N`, can be at most 1800 (30 minutes).

            Waypoint waiting times, setup times or anything other than driving time does not
            cause power consumption but function curve keeps advancing.

            The list can have a maximum of six `TIME`,`POWER` pairs.

            **Note**: When using this feature `departureTime` needs to be set and cannot be set to `any`.

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
    """

    drive_efficiency: float
    recuperation_efficiency: float
    auxiliary_power_consumption: float | Unset = UNSET
    air_density: float | Unset = 1.225
    auxiliary_power_curve: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        drive_efficiency = self.drive_efficiency

        recuperation_efficiency = self.recuperation_efficiency

        auxiliary_power_consumption = self.auxiliary_power_consumption

        air_density = self.air_density

        auxiliary_power_curve = self.auxiliary_power_curve

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "driveEfficiency": drive_efficiency,
            "recuperationEfficiency": recuperation_efficiency,
        })
        if auxiliary_power_consumption is not UNSET:
            field_dict["auxiliaryPowerConsumption"] = auxiliary_power_consumption
        if air_density is not UNSET:
            field_dict["airDensity"] = air_density
        if auxiliary_power_curve is not UNSET:
            field_dict["auxiliaryPowerCurve"] = auxiliary_power_curve

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        drive_efficiency = d.pop("driveEfficiency")

        recuperation_efficiency = d.pop("recuperationEfficiency")

        auxiliary_power_consumption = d.pop("auxiliaryPowerConsumption", UNSET)

        air_density = d.pop("airDensity", UNSET)

        auxiliary_power_curve = d.pop("auxiliaryPowerCurve", UNSET)

        base_ev_physical_model = cls(
            drive_efficiency=drive_efficiency,
            recuperation_efficiency=recuperation_efficiency,
            auxiliary_power_consumption=auxiliary_power_consumption,
            air_density=air_density,
            auxiliary_power_curve=auxiliary_power_curve,
        )

        base_ev_physical_model.additional_properties = d
        return base_ev_physical_model

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="VehicleRestrictionMaxWeight")


@_attrs_define
class VehicleRestrictionMaxWeight:
    """Contains the maximum permitted weight, specified in kilograms, along with the specific type of the maximum permitted
    weight restriction.

        Attributes:
            value (int | Unset):
            type_ (str | Unset): Extensible enum: `empty` `gross` `current` `unknown` `...`
                Represents the specific type of the maximum permitted weight restriction.
                * `empty`: Restriction is for empty weight of the vehicle combination.
                * `gross`: Restriction is for gross weight.
                * `current`: Restriction is for current weight.
                * `unknown`: Restriction may apply to gross or current weight. Specific `type` data for the restriction is not
                available.

                **NOTES:**
                * A restriction of type `unknown` may change to `gross` or `current` when data becomes available in future.
                * A restriction of type `gross` or `current` may also change to a different type if actual regulation changes.
    """

    value: int | Unset = UNSET
    type_: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        value = self.value

        type_ = self.type_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if value is not UNSET:
            field_dict["value"] = value
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        value = d.pop("value", UNSET)

        type_ = d.pop("type", UNSET)

        vehicle_restriction_max_weight = cls(
            value=value,
            type_=type_,
        )

        vehicle_restriction_max_weight.additional_properties = d
        return vehicle_restriction_max_weight

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class Return(str, Enum):
    ACTIONS = "actions"
    ELEVATION = "elevation"
    INCIDENTS = "incidents"
    INSTRUCTIONS = "instructions"
    MLDURATION = "mlDuration"
    NOTHROUGHRESTRICTIONS = "noThroughRestrictions"
    PASSTHROUGH = "passthrough"
    POLYLINE = "polyline"
    POTENTIALTIMEDEPENDENTVIOLATIONS = "potentialTimeDependentViolations"
    ROUTEHANDLE = "routeHandle"
    ROUTELABELS = "routeLabels"
    ROUTINGZONES = "routingZones"
    SUMMARY = "summary"
    TOLLS = "tolls"
    TRAVELSUMMARY = "travelSummary"
    TRUCKROADTYPES = "truckRoadTypes"
    TURNBYTURNACTIONS = "turnByTurnActions"
    TYPICALDURATION = "typicalDuration"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.truck_fuel_maximum_truck_class import TruckFuelMaximumTruckClass
from ..models.truck_fuel_type import TruckFuelType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.fuel_price import FuelPrice


T = TypeVar("T", bound="TruckFuel")


@_attrs_define
class TruckFuel:
    """
    Attributes:
        type_ (TruckFuelType): **BETA, RESTRICTED**

            The type of truck fuel

            Description of supported values:

            - **BETA, RESTRICTED** `truck_cng`: compressed natural gas (CNG) fuel for truck
            - **BETA, RESTRICTED** `truck_diesel`: diesel fuel for truck
            - **BETA, RESTRICTED** `truck_hydrogen`: hydrogen fuel for truck
            - **BETA, RESTRICTED** `truck_lng`: liquefied natural gas (LNG) fuel for truck
        available (bool): **BETA, RESTRICTED**

            When set to:
            - `true` indicates that this type of truck fuel is available at this place
            - `false` indicates that this type of truck fuel is not available at this place
        maximum_truck_class (TruckFuelMaximumTruckClass | Unset): **BETA, RESTRICTED**

            The truck classes

            Description of supported values:

            - **BETA, RESTRICTED** `heavy`: Medium and Heavy trucks are allowed to fuel in the gas station
            - **BETA, RESTRICTED** `medium`: Only medium size truck is allowed to fuel in the gas station
        price (FuelPrice | Unset):
        large_nozzle (bool | Unset): **BETA, RESTRICTED**

            When set to:
            - `true`: indicates that this Place has Truck diesel pumps with large nozzles
            - `false`: indicates that this Place does not have Truck diesel pumps with large nozzles
    """

    type_: TruckFuelType
    available: bool
    maximum_truck_class: TruckFuelMaximumTruckClass | Unset = UNSET
    price: FuelPrice | Unset = UNSET
    large_nozzle: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        available = self.available

        maximum_truck_class: str | Unset = UNSET
        if not isinstance(self.maximum_truck_class, Unset):
            maximum_truck_class = self.maximum_truck_class.value

        price: dict[str, Any] | Unset = UNSET
        if not isinstance(self.price, Unset):
            price = self.price.to_dict()

        large_nozzle = self.large_nozzle

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "available": available,
        })
        if maximum_truck_class is not UNSET:
            field_dict["maximumTruckClass"] = maximum_truck_class
        if price is not UNSET:
            field_dict["price"] = price
        if large_nozzle is not UNSET:
            field_dict["largeNozzle"] = large_nozzle

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.fuel_price import FuelPrice

        d = dict(src_dict)
        type_ = TruckFuelType(d.pop("type"))

        available = d.pop("available")

        _maximum_truck_class = d.pop("maximumTruckClass", UNSET)
        maximum_truck_class: TruckFuelMaximumTruckClass | Unset
        if isinstance(_maximum_truck_class, Unset):
            maximum_truck_class = UNSET
        else:
            maximum_truck_class = TruckFuelMaximumTruckClass(_maximum_truck_class)

        _price = d.pop("price", UNSET)
        price: FuelPrice | Unset
        if isinstance(_price, Unset):
            price = UNSET
        else:
            price = FuelPrice.from_dict(_price)

        large_nozzle = d.pop("largeNozzle", UNSET)

        truck_fuel = cls(
            type_=type_,
            available=available,
            maximum_truck_class=maximum_truck_class,
            price=price,
            large_nozzle=large_nozzle,
        )

        truck_fuel.additional_properties = d
        return truck_fuel

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.autosuggest_entity_result_item import AutosuggestEntityResultItem
    from ..models.autosuggest_query_result_item import AutosuggestQueryResultItem
    from ..models.query_term_result_item import QueryTermResultItem


T = TypeVar("T", bound="OpenSearchAutosuggestResponse")


@_attrs_define
class OpenSearchAutosuggestResponse:
    """
    Attributes:
        items (list[AutosuggestEntityResultItem | AutosuggestQueryResultItem]): If the `offset` request parameter is not
            set, `items` contains a *full list* of JSON objects with a payload relevant to the request.
            The ranking order is from most to least relevant and is based on the matched location criteria.

            If the `offset` request parameter is set, `items` contains a contiguous subset of objects from the *full list*
            described before: A page of `<count>` objects at a certain `<offset>` value from the beginning of the *full
            list*.
        query_terms (list[QueryTermResultItem]): Suggestions for refining individual query terms
        offset (int | Unset): **ALPHA**

            The `offset` value from the full list from which the page is taken.

            This is expected to reflect the `offset` request parameter value
        next_offset (int | Unset): **ALPHA**

            The `offset` value of the next page. This element is omitted on the last page.

            This response element is only set when in pagination mode (when the query parameter `offset` is set).
        count (int | Unset): **ALPHA**

            The actual page size: The number of objects returned in the `items` array.

            This response element is only set when in pagination mode (when the query parameter `offset` is set).
        limit (int | Unset): **ALPHA**

            Maximum number of objects in `items` as specified in request.

            This response element is only set when in pagination mode (when the query parameter `offset` is set).
    """

    items: list[AutosuggestEntityResultItem | AutosuggestQueryResultItem]
    query_terms: list[QueryTermResultItem]
    offset: int | Unset = UNSET
    next_offset: int | Unset = UNSET
    count: int | Unset = UNSET
    limit: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.autosuggest_entity_result_item import AutosuggestEntityResultItem

        items = []
        for items_item_data in self.items:
            items_item: dict[str, Any]
            if isinstance(items_item_data, AutosuggestEntityResultItem):
                items_item = items_item_data.to_dict()
            else:
                items_item = items_item_data.to_dict()

            items.append(items_item)

        query_terms = []
        for query_terms_item_data in self.query_terms:
            query_terms_item = query_terms_item_data.to_dict()
            query_terms.append(query_terms_item)

        offset = self.offset

        next_offset = self.next_offset

        count = self.count

        limit = self.limit

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "items": items,
            "queryTerms": query_terms,
        })
        if offset is not UNSET:
            field_dict["offset"] = offset
        if next_offset is not UNSET:
            field_dict["nextOffset"] = next_offset
        if count is not UNSET:
            field_dict["count"] = count
        if limit is not UNSET:
            field_dict["limit"] = limit

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.autosuggest_entity_result_item import AutosuggestEntityResultItem
        from ..models.autosuggest_query_result_item import AutosuggestQueryResultItem
        from ..models.query_term_result_item import QueryTermResultItem

        d = dict(src_dict)
        items = []
        _items = d.pop("items")
        for items_item_data in _items:

            def _parse_items_item(
                data: object,
            ) -> AutosuggestEntityResultItem | AutosuggestQueryResultItem:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    items_item_type_0 = AutosuggestEntityResultItem.from_dict(data)

                    return items_item_type_0
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                items_item_type_1 = AutosuggestQueryResultItem.from_dict(data)

                return items_item_type_1

            items_item = _parse_items_item(items_item_data)

            items.append(items_item)

        query_terms = []
        _query_terms = d.pop("queryTerms")
        for query_terms_item_data in _query_terms:
            query_terms_item = QueryTermResultItem.from_dict(query_terms_item_data)

            query_terms.append(query_terms_item)

        offset = d.pop("offset", UNSET)

        next_offset = d.pop("nextOffset", UNSET)

        count = d.pop("count", UNSET)

        limit = d.pop("limit", UNSET)

        open_search_autosuggest_response = cls(
            items=items,
            query_terms=query_terms,
            offset=offset,
            next_offset=next_offset,
            count=count,
            limit=limit,
        )

        open_search_autosuggest_response.additional_properties = d
        return open_search_autosuggest_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.route_response_notice import RouteResponseNotice
    from ..models.router_route import RouterRoute


T = TypeVar("T", bound="RouterRouteResponse")


@_attrs_define
class RouterRouteResponse:
    """Returns a list of routes.

    Attributes:
        routes (list[RouterRoute]): List of possible routes
        notices (list[RouteResponseNotice] | Unset): Contains a list of issues related to this route calculation.
            Please refer to the `code` attribute for possible values.
    """

    routes: list[RouterRoute]
    notices: list[RouteResponseNotice] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        routes = []
        for routes_item_data in self.routes:
            routes_item = routes_item_data.to_dict()
            routes.append(routes_item)

        notices: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.notices, Unset):
            notices = []
            for notices_item_data in self.notices:
                notices_item = notices_item_data.to_dict()
                notices.append(notices_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "routes": routes,
        })
        if notices is not UNSET:
            field_dict["notices"] = notices

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.route_response_notice import RouteResponseNotice
        from ..models.router_route import RouterRoute

        d = dict(src_dict)
        routes = []
        _routes = d.pop("routes")
        for routes_item_data in _routes:
            routes_item = RouterRoute.from_dict(routes_item_data)

            routes.append(routes_item)

        _notices = d.pop("notices", UNSET)
        notices: list[RouteResponseNotice] | Unset = UNSET
        if _notices is not UNSET:
            notices = []
            for notices_item_data in _notices:
                notices_item = RouteResponseNotice.from_dict(notices_item_data)

                notices.append(notices_item)

        router_route_response = cls(
            routes=routes,
            notices=notices,
        )

        router_route_response.additional_properties = d
        return router_route_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="AxleGroupWeight")


@_attrs_define
class AxleGroupWeight:
    """Contains the maximum allowed weight for an axle-group.

    Attributes:
        max_weight (int | Unset): Maximum weight of the axle-group-weight restriction, in kilograms.
        axle_group (str | Unset): Extensible enum: `single` `tandem` `triple` `quad` `quint` `...`
            Axle-group associated with the restriction.

            Possible values are:

            * single
            * tandem
            * triple
            * quad
            * quint
    """

    max_weight: int | Unset = UNSET
    axle_group: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        max_weight = self.max_weight

        axle_group = self.axle_group

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if max_weight is not UNSET:
            field_dict["maxWeight"] = max_weight
        if axle_group is not UNSET:
            field_dict["axleGroup"] = axle_group

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        max_weight = d.pop("maxWeight", UNSET)

        axle_group = d.pop("axleGroup", UNSET)

        axle_group_weight = cls(
            max_weight=max_weight,
            axle_group=axle_group,
        )

        axle_group_weight.additional_properties = d
        return axle_group_weight

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.address_highlighting_information import AddressHighlightingInformation
    from ..models.range_ import Range


T = TypeVar("T", bound="TitleAndAddressHighlighting")


@_attrs_define
class TitleAndAddressHighlighting:
    """
    Attributes:
        title (list[Range] | Unset): Ranges of indexes that matched in the title attribute
        address (AddressHighlightingInformation | Unset):
    """

    title: list[Range] | Unset = UNSET
    address: AddressHighlightingInformation | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        title: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.title, Unset):
            title = []
            for title_item_data in self.title:
                title_item = title_item_data.to_dict()
                title.append(title_item)

        address: dict[str, Any] | Unset = UNSET
        if not isinstance(self.address, Unset):
            address = self.address.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if title is not UNSET:
            field_dict["title"] = title
        if address is not UNSET:
            field_dict["address"] = address

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.address_highlighting_information import (
            AddressHighlightingInformation,
        )
        from ..models.range_ import Range

        d = dict(src_dict)
        _title = d.pop("title", UNSET)
        title: list[Range] | Unset = UNSET
        if _title is not UNSET:
            title = []
            for title_item_data in _title:
                title_item = Range.from_dict(title_item_data)

                title.append(title_item)

        _address = d.pop("address", UNSET)
        address: AddressHighlightingInformation | Unset
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = AddressHighlightingInformation.from_dict(_address)

        title_and_address_highlighting = cls(
            title=title,
            address=address,
        )

        title_and_address_highlighting.additional_properties = d
        return title_and_address_highlighting

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.field_score import FieldScore


T = TypeVar("T", bound="Scoring")


@_attrs_define
class Scoring:
    """
    Attributes:
        query_score (float | Unset): Indicates how good the input matches the returned address. It is equal to 1 if all
            input tokens are recognized and matched.
        field_score (FieldScore | Unset):
    """

    query_score: float | Unset = UNSET
    field_score: FieldScore | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        query_score = self.query_score

        field_score: dict[str, Any] | Unset = UNSET
        if not isinstance(self.field_score, Unset):
            field_score = self.field_score.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if query_score is not UNSET:
            field_dict["queryScore"] = query_score
        if field_score is not UNSET:
            field_dict["fieldScore"] = field_score

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.field_score import FieldScore

        d = dict(src_dict)
        query_score = d.pop("queryScore", UNSET)

        _field_score = d.pop("fieldScore", UNSET)
        field_score: FieldScore | Unset
        if isinstance(_field_score, Unset):
            field_score = UNSET
        else:
            field_score = FieldScore.from_dict(_field_score)

        scoring = cls(
            query_score=query_score,
            field_score=field_score,
        )

        scoring.additional_properties = d
        return scoring

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="EMobilityServiceProvider")


@_attrs_define
class EMobilityServiceProvider:
    """
    Attributes:
        name (str): The name of the e-Mobility Service Provider. The eMSP name may change but the respective partner ID
            will remain stable.
        partner_id (str): The unique and stable ID for the e-Mobility Service Provider provided by HERE platform.
            The partner ID can be used to display EV stations supported by eMSP for which drivers have a subscription.
            The list of eMSP partner IDs can be retrieved using the EV Charge Points API
            [`roaming`](https://www.here.com/docs/bundle/ev-charge-points-api-developer-guide/page/topics/resource-
            roamings.html) resource.
    """

    name: str
    partner_id: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        partner_id = self.partner_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "name": name,
            "partnerId": partner_id,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        partner_id = d.pop("partnerId")

        e_mobility_service_provider = cls(
            name=name,
            partner_id=partner_id,
        )

        e_mobility_service_provider.additional_properties = d
        return e_mobility_service_provider

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Avoid")


@_attrs_define
class Avoid:
    """
    Attributes:
        features (str | Unset): A comma-separated list of features that routes should avoid.

            * `seasonalClosure`
            * `tollRoad`: This option avoids roads that have an applicable toll.
            * `controlledAccessHighway`
            * `ferry`
            * `carShuttleTrain`
            * `tunnel`
            * `dirtRoad`
            * `difficultTurns`: This option avoids difficult turns, sharp turns and U-turns on multi-digitized roads,
            intersections and stopover-type waypoints. It is only supported for the `truck` transport mode.
              **Disclaimer: This parameter is currently in beta release. Depending on your location and vehicle type, the
            outcome might not be according to the expectations. Please test the behavior of this feature for your use case
            before enabling it.**
            * `uTurns`: This option avoids U-turns on multi-digitized roads, intersections, stopover-type waypoints. U-turns
            on mini roundabouts (roundabouts with a small turn radius, which are difficult for long vehicles to navigate)
            are avoided when `transportMode = truck`. This option is not supported for pedestrian, bicycle and scooter
            transport modes.

            **Notes**
              - It may not always be possible to find a route which avoids the requested feature. A route may not be
            returned, or one may be returned with a critical notice.
              - Using `avoid[features]=tollRoad` does not optimize the route to reduce the overall toll costs.
              - Using `avoid[features]=tollRoad` is not recommended for large vehicles, e.g., trucks or buses. Even if a
            route is suggested, it may use smaller roads, where driving such vehicles is not convenient or safe.
              - Using `avoid[features]=difficultTurns` together with `avoid[features]=controlledAccessHighway` or
            `avoid[features]=tollRoads` is not recommended, as it may lead to routes which could include difficult turns.
        areas (str | Unset): A pipe separated list of user-defined areas that routes should avoid/exclude going through.

            Notes:
            * Maximum count of avoided and excluded polygons and corridors is 20.
            * Maximum total count of avoided and excluded bounding boxes, polygons, corridors, including exceptions, is 250.

            Format: `{area1}[!exception1.1[!exception1.2...]]|{area2}[!exception2.1[!exception2.2...]]|{area3}[!exception3.1
            [!exception3.2...]]...`

            Supported areas:
            * Bounding box - A rectangular area on earth defined by a comma separated list of two latitude and two longitude
            values.
              - Format: `bbox:{west},{south},{east},{north}`
                + `{west}`  - Longitude value of the westernmost point of the area.
                + `{south}` - Latitude value of the southernmost point of the area.
                + `{east}`  - Longitude value of the easternmost point of the area.
                + `{north}` - Latitude value of the northernmost point of the area.
              - Example: `bbox:13.082,52.416,13.628,52.626` - Bounding box of Berlin
            * Polygon - A polygon on earth which defines area.
              Possible formats:
              1) As list of geopoints.
                - Format: `polygon:{lat},{lon};{lat},{lon};{lat},{lon}...`
                  + `{lat}` - Latitude
                  + `{lon}` - Longitude
                - Example: `polygon:52.416,13.082;52.626,13.628;52.916,13.482` - Polygon in Berlin
              2) As [Flexible Polyline](https://github.com/heremaps/flexible-polyline) Encoding.
                - Support only 2D polyline (without `elevation` specified).
                - Format: `polygon:{encoded_polyline}`
                  + `{encoded_polyline}` - encoded [Flexible Polyline](https://github.com/heremaps/flexible-polyline)
                - Example: `polygon:BF05xgKuy2xCx9B7vUl0OhnR54EqSzpEl-HxjD3pBiGnyGi2CvwFsgD3nD4vB6e`
            * Corridor - A polyline with a specified radius (integer, in meters) that defines width of corridor area.
              Possible formats:
              1) As a list of geopoints that defines a polyline and a radius that defines an area around the polyline.
                - Format: `corridor:{lat},{lon};{lat},{lon};{lat},{lon}...;r={radius}`
                - Example: `corridor:52.416,13.082;52.626,13.628;52.916,13.482;r=1000`
              2) As [Flexible Polyline](https://github.com/heremaps/flexible-polyline) encoding and a radius that defines an
            area around the polyline.
                - Supports only 2D polyline (without `elevation` specified).
                - Format: `corridor:{encoded_polyline};r={radius}`
                  + `{encoded_polyline}` - encoded [Flexible Polyline](https://github.com/heremaps/flexible-polyline)
                - Example: `corridor:BF05xgKuy2xCx9B7vUl0OhnR54EqSzpEl-HxjD3pBiGnyGi2CvwFsgD3nD4vB6e;r=1000`

            * Exception - area to exclude from avoidance. Any area type can be specified. Any area type can be used as an
            exception. Multiple exceptions can be specified for each area.
              - Format: `{area to avoid/exclude}!exception={area}`
              - Example: `bbox:13.082,52.416,13.628,52.626!exception=polygon:BF05xgKuy2xCx9B7vUl0OhnR54EqSzpEl-
            HxjD3pBiGnyGi2CvwFsgD3nD4vB6e!exception=bbox:13.082,52.416,13.628,52.626`

              Notes:
              * Maximum count of avoided and excluded polygons and corridors is 20.
              * Maximum total count of avoided and excluded bounding boxes, polygons, corridors, including exceptions, is
            250.
              * Minimum count of coordinates in one polygon is 3
              * Minimum count of coordinates in any single corridor is 2.
              * Maximum count of coordinates in one polygon or in one corridor is 100
              * The polygon is closed automatically, there is no need to duplicate the first point as the last one.
              * Self-intersecting polygons are not supported.
        segments (str | Unset): A comma separated list of segment identifiers that routes should avoid going through.

            Each entry has the following structure:
            `{segmentId}(#{direction})?`

            The individual parts are:
            * segmentId: The identifier of the referenced topology segment inside the catalog, example:
            `here:cm:segment:207551710`
            * direction (optional): Either '*' for bidirectional (default), '+' for positive direction, or '-' for negative
            direction

            Example of a parameter value excluding two segments:
            `here:cm:segment:207551710#+,here:cm:segment:76771992#*`

            **Note**: Maximum number of penalized segments in one request should not be greater than 1000.
                      "Penalized segments" refers to segments that either have a restriction on maximum baseSpeed with
            `maxSpeedOnSegment`
                      or avoided with `avoid[segments]`
        zone_categories (str | Unset): Specifies a list of categories of zones which routes should avoid going through.

            Format: `Categories[ZoneCategoryOptions]`

            * Categories: `{cat1},{cat2}...`
              A comma separated list of zone categories.
            * ZoneCategoriesOptions (optional): `;option1=value1;options2=value2...`
              A list of options for zone categories in `KEY=VALUE` form.

            Supported zone category options:
              * exceptZoneIds: A comma-separated list of zone identifiers, which should not be taken into account for
            evaluation of zone categories to avoid.

            Supported zone categories:
              * `vignette`
              * `congestionPricing`
              * `environmental`

            **Note**: Zones that don't apply to the current vehicle aren't avoided.
            Time-dependent zones are avoided only during the validity period.

            Example of zone categories avoidance:
            `avoid[zoneCategories]=environmental,vignette`

            Example of zone categories avoidance with exceptions:
            `avoid[zoneCategories]=environmental,vignette;exceptZoneIds=here:cm:envzone:3`
        zone_identifiers (str | Unset): A comma separated list containing identifiers of zones that routes should avoid
            going through.

            **Note**: Zones specified by id in this manner are avoided even if their conditions don't apply to the current
            vehicle.
            Time-dependent zones are avoided by id even outside of the validity period.

            Example of an identifier referencing an environmental zone:
            `here:cm:envzone:2`
        truck_road_types (str | Unset): A comma-separated list of truck road type identifiers to be avoided.

            A truck road type is an identifier associated with roads that have additional regulations applied by local
            administration for traversal by heavy vehicles like trucks.
            For example, the BK Bearing Class regulations in Sweden, and ET categories in Mexico.
            Identifiers for supported truck road types are specified in HERE Map Content
            [TruckRoadType](https://www.here.com/docs/bundle/map-content-schema-data-
            specification/page/topics_schema/truckroadtypeattribute.html).

            Example: `avoid[truckRoadTypes]=BK1,BK2,BK3,BK4`
             Example: BK1,BK2,BK3,BK4.
        toll_transponders (str | Unset): Specifies that routes should avoid roads where the specified toll transponders
            are the only payment method.

            The value of the parameter is a comma-separated list of transponder systems that the user has. Alternatively,
            the user can also specify `all` as a list element to state they have all required transponders along any
            potential route.

            **Note**: Currently, the only valid value is `all`.
            Example: `avoid[tollTransponders]=all`
    """

    features: str | Unset = UNSET
    areas: str | Unset = UNSET
    segments: str | Unset = UNSET
    zone_categories: str | Unset = UNSET
    zone_identifiers: str | Unset = UNSET
    truck_road_types: str | Unset = UNSET
    toll_transponders: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        features = self.features

        areas = self.areas

        segments = self.segments

        zone_categories = self.zone_categories

        zone_identifiers = self.zone_identifiers

        truck_road_types = self.truck_road_types

        toll_transponders = self.toll_transponders

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if features is not UNSET:
            field_dict["features"] = features
        if areas is not UNSET:
            field_dict["areas"] = areas
        if segments is not UNSET:
            field_dict["segments"] = segments
        if zone_categories is not UNSET:
            field_dict["zoneCategories"] = zone_categories
        if zone_identifiers is not UNSET:
            field_dict["zoneIdentifiers"] = zone_identifiers
        if truck_road_types is not UNSET:
            field_dict["truckRoadTypes"] = truck_road_types
        if toll_transponders is not UNSET:
            field_dict["tollTransponders"] = toll_transponders

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        features = d.pop("features", UNSET)

        areas = d.pop("areas", UNSET)

        segments = d.pop("segments", UNSET)

        zone_categories = d.pop("zoneCategories", UNSET)

        zone_identifiers = d.pop("zoneIdentifiers", UNSET)

        truck_road_types = d.pop("truckRoadTypes", UNSET)

        toll_transponders = d.pop("tollTransponders", UNSET)

        avoid = cls(
            features=features,
            areas=areas,
            segments=segments,
            zone_categories=zone_categories,
            zone_identifiers=zone_identifiers,
            truck_road_types=truck_road_types,
            toll_transponders=toll_transponders,
        )

        avoid.additional_properties = d
        return avoid

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PostMultiRevgeocodeShowRelatedItem(str, Enum):
    INTERSECTIONS = "intersections"
    NEARBYADDRESS = "nearbyAddress"
    PARENTPA = "parentPA"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.tripadvisor_image_variant import TripadvisorImageVariant


T = TypeVar("T", bound="TripadvisorImageVariants")


@_attrs_define
class TripadvisorImageVariants:
    """
    Attributes:
        thumbnail (TripadvisorImageVariant | Unset):
        small (TripadvisorImageVariant | Unset):
        medium (TripadvisorImageVariant | Unset):
        large (TripadvisorImageVariant | Unset):
        original (TripadvisorImageVariant | Unset):
    """

    thumbnail: TripadvisorImageVariant | Unset = UNSET
    small: TripadvisorImageVariant | Unset = UNSET
    medium: TripadvisorImageVariant | Unset = UNSET
    large: TripadvisorImageVariant | Unset = UNSET
    original: TripadvisorImageVariant | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        thumbnail: dict[str, Any] | Unset = UNSET
        if not isinstance(self.thumbnail, Unset):
            thumbnail = self.thumbnail.to_dict()

        small: dict[str, Any] | Unset = UNSET
        if not isinstance(self.small, Unset):
            small = self.small.to_dict()

        medium: dict[str, Any] | Unset = UNSET
        if not isinstance(self.medium, Unset):
            medium = self.medium.to_dict()

        large: dict[str, Any] | Unset = UNSET
        if not isinstance(self.large, Unset):
            large = self.large.to_dict()

        original: dict[str, Any] | Unset = UNSET
        if not isinstance(self.original, Unset):
            original = self.original.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if thumbnail is not UNSET:
            field_dict["thumbnail"] = thumbnail
        if small is not UNSET:
            field_dict["small"] = small
        if medium is not UNSET:
            field_dict["medium"] = medium
        if large is not UNSET:
            field_dict["large"] = large
        if original is not UNSET:
            field_dict["original"] = original

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.tripadvisor_image_variant import TripadvisorImageVariant

        d = dict(src_dict)
        _thumbnail = d.pop("thumbnail", UNSET)
        thumbnail: TripadvisorImageVariant | Unset
        if isinstance(_thumbnail, Unset):
            thumbnail = UNSET
        else:
            thumbnail = TripadvisorImageVariant.from_dict(_thumbnail)

        _small = d.pop("small", UNSET)
        small: TripadvisorImageVariant | Unset
        if isinstance(_small, Unset):
            small = UNSET
        else:
            small = TripadvisorImageVariant.from_dict(_small)

        _medium = d.pop("medium", UNSET)
        medium: TripadvisorImageVariant | Unset
        if isinstance(_medium, Unset):
            medium = UNSET
        else:
            medium = TripadvisorImageVariant.from_dict(_medium)

        _large = d.pop("large", UNSET)
        large: TripadvisorImageVariant | Unset
        if isinstance(_large, Unset):
            large = UNSET
        else:
            large = TripadvisorImageVariant.from_dict(_large)

        _original = d.pop("original", UNSET)
        original: TripadvisorImageVariant | Unset
        if isinstance(_original, Unset):
            original = UNSET
        else:
            original = TripadvisorImageVariant.from_dict(_original)

        tripadvisor_image_variants = cls(
            thumbnail=thumbnail,
            small=small,
            medium=medium,
            large=large,
            original=original,
        )

        tripadvisor_image_variants.additional_properties = d
        return tripadvisor_image_variants

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Tolls")


@_attrs_define
class Tolls:
    """Vehicle-independent options that may affect route toll calculation as well as options
    affecting the output of the tolls, such as summaries.

    Since this parameter controls behaviour related to tolls in the return part of the response,
    use of this parameter requires `return=tolls` to be selected.

        Attributes:
            transponders (str | Unset): Certain toll systems allow users to pay for the usage of the corresponding tollroads
                using transponders.
                As the price that the user pays with a transponder could be different from other payment methods for accessing
                the same toll roads
                this  is provided to allow the user to specify the transponders that they have.
                If a toll system requires a certain transponder to access it and the user states that they have it,
                the price for payment with transponders will be used when reporting fare prices
                and for summaries, if required by the `tolls[summaries]` parameter.

                The value of the parameter is a comma-separated list of transponder systems that the user has. Alternatively,
                the user can also specify `all` as a list element to state they have all required transponders along any
                potential route.

                **Note**: Currently, the only valid value is `all`.
            vignettes (str | Unset): This parameter allows the user to specify for which toll roads the user has valid
                vignettes.
                If a road requires a certain vignette and the user states that they have it, no notices will be
                given regarding the requirement to have it.

                The value of the parameter is a comma-separated list of vignettes that the user has. Alternatively,
                the user can also specify `all` as a list element to state they have all required vignettes along any potential
                route.

                No toll costs information will be returned for a given road requiring a vignette if the user states
                they already have it, as no further payment is necessary. If `tolls` are requested for spans, the toll
                sections for these types of toll systems are still reported, nevertheless.

                **Note**: Currently, the only valid value is `all`.
            summaries (list[str] | Unset): Items extensible enum: `total` `tollSystem` `country` `...`
                This parameter allows the user to specify criteria for tolls aggregation.
                Multiple values may be requested at once.
                Toll aggregation is performed at the section level only.

                Possible values are:
                  - `total`: the user wants a single value summarizing the tolls to be paid in the section.
                    This summary criterion requires that a `currency` has been passed as a parameter, to group
                    multi-currency roads together, even if the route would traverse roads that use only one currency;
                    see `currency` parameter.
                  - `tollSystem`: toll costs are aggregated per toll system.
                  - `country`: toll costs are aggregated per country.

                Note that any toll instance may have multiple prices, depending on factors such as time of day,
                payment methods, etc. that are not available in the request. As a result,
                the most economical value is selected for summary calculation, so summaries should be considered
                informative only.
            vehicle_category (str | Unset): Extensible enum: `minibus` `...`
                Defines special toll vehicle types. Usual types like car or truck are determined from transport mode.

                | category  | Description |
                | --------- | ------- |
                | minibus | Commercial buses with a seating capacity of 16-25 passengers (NA) or a small bus that is used to
                transport a maximum of 15 passengers. Can be used only with transport mode `car` |

                **NOTE:** It can be extended by other vehicle categories in the future.
            emission_type (str | Unset): Defines the emission types and CO2 emission classes as defined by the toll
                operator.
                The emission types defined are based on the Emission standards. Emission types are only published when the toll
                cost is defined based on emission type classes.

                ## Format

                Format: `EmissionType[CO2EmissionClass]`

                `EmissionType` is specified as `euro6`, `euro5`, etc. Allowed values are `[euro1, euro2, euro3, euro4, euro5,
                euro6, euroEev]`.

                `CO2EmissionClass` is optional and is specified as `;co2class=1`, `;co2class=2`, etc. Allowed values for
                `co2class` are `[1, 2, 3, 4, 5]`.

                ## Examples:

                Only toll emission type (euro6), without CO2 emission class: `euro6`

                Toll emission type (euro6) with CO2 emission class (class1): `euro6;co2class=1`

                **NOTES:**
                * This parameter is not compatible with EV routing. When EV routing is used (i.e., any `ev` namespace parameter
                is specified in the request or `vehicle[engineType]=electric` is specified), the appropriate emission type
                (Electric Vehicle) is used.
                * For cases other than EV routing: when not specified, `EmissionType` defaults to `euro5`. If `EmissionType` is
                provided without a `co2class`, then `co2class` defaults to `1`.
                * Providing only a `CO2EmissionClass` without an `EmissionType` is not supported, i.e. `euro6;co2class=1` is
                valid but `co2class=1` is invalid.
                * Providing an invalid combination of `EmissionType` and `CO2EmissionClass` as input may result in unexpected
                tolls.
                 Example: euro6.
    """

    transponders: str | Unset = UNSET
    vignettes: str | Unset = UNSET
    summaries: list[str] | Unset = UNSET
    vehicle_category: str | Unset = UNSET
    emission_type: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        transponders = self.transponders

        vignettes = self.vignettes

        summaries: list[str] | Unset = UNSET
        if not isinstance(self.summaries, Unset):
            summaries = self.summaries

        vehicle_category = self.vehicle_category

        emission_type = self.emission_type

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if transponders is not UNSET:
            field_dict["transponders"] = transponders
        if vignettes is not UNSET:
            field_dict["vignettes"] = vignettes
        if summaries is not UNSET:
            field_dict["summaries"] = summaries
        if vehicle_category is not UNSET:
            field_dict["vehicleCategory"] = vehicle_category
        if emission_type is not UNSET:
            field_dict["emissionType"] = emission_type

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        transponders = d.pop("transponders", UNSET)

        vignettes = d.pop("vignettes", UNSET)

        summaries = cast(list[str], d.pop("summaries", UNSET))

        vehicle_category = d.pop("vehicleCategory", UNSET)

        emission_type = d.pop("emissionType", UNSET)

        tolls = cls(
            transponders=transponders,
            vignettes=vignettes,
            summaries=summaries,
            vehicle_category=vehicle_category,
            emission_type=emission_type,
        )

        tolls.additional_properties = d
        return tolls

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="RoutingZone")


@_attrs_define
class RoutingZone:
    r"""Information about a routing zone.

    Attributes:
        ref (str | Unset): A reference to a routing zone in HMC.

            The standard representation of a routing zone reference has the following structure:
            `{catalogHrn}:{catalogVersion}:({layerId})?:{tileId}:{zoneId}`

            The individual parts are:
            * `catalogHrn`: The HERE Resource Name that identifies the source catalog of the routing zone, example:
            `hrn:here:data::olp-here:rib-2`
            * `catalogVersion`: The catalog version
            * `layerId` (optional): The layer inside the catalog where the routing zone is located, example: `environmental-
            zones`
            * `tileId`: The HERE tile key of the partition/tile where the routing zone is located in the given version of
            the catalog
            * `zoneId`: The identifier of the referenced routing zone within the catalog, example: `here:cm:envzone:3455277`

            Example of a reference to an environmental zone in standard form:
            `hrn:here:data::olp-here:rib-2:1557:environmental-zones:all:here:cm:envzone:3455277`

            In order to reduce response size, routing zone references can also be provided in a compact representation.
            In compact form, parts of a reference are replaced by placeholders, which can be resolved using the
            `refReplacements` dictionary in the parent section.
            The placeholder format is ```\$\d+``` and needs to be surrounded by colons or string start/end. It can be
            captured with the following regular expression: ```(^|:)\$\d+(:|$)``` .

            Example of the aforementioned environmental zone reference in compact form: `$0:$1:3455277`
            With the corresponding `refReplacements`:
            ```
            "refReplacements": {
              "0": "hrn:here:data::olp-here:rib-2:1557",
              "1": "environmental-zones:all:here:cm:envzone"
            }
            ```
        type_ (str | Unset): Extensible enum: `environmental` `vignette` `...`
            The type of the routing zone. A routing zone is defined based on an underlying feature/resource.
            The standard representation of a routing zone reference is
            `{catalogHrn}:{catalogVersion}:({layerId})?:{tileId}:{zoneId}` (refer to `RoutingZoneReference` description in
            this document for details).
            `{zoneId}` represents the identifier of the underlying feature/resource within the catalog, in the
            `"domain:system:type:id"` format.
            This attribute corresponds to the `"domain:system:type"` portion of the identifier.
            E.g. `type=environmental` corresponds to the `"here:cm:envzone"` portion of the identifiers of all environmental
            zones within a catalog.

            To further distinguish between the sub-categories of the `type` (if applicable) use the `category` attribute.

            NOTE: To maintain legacy support `type=vignette` is not renamed but corresponds to the `"here:cm:tollsystem"`
            portion of the identifier for all toll cost features.
        name (str | Unset): The routing zone's name.
        category (str | Unset): Extensible enum: `environmental` `vignette` `congestionPricing` `...`
            The category of the routing zone. Corresponds to the sub-category (if applicable) of the feature/resource
            defining a routing zone, e.g. `vignette` and `congestionPricing` sub-categories of the toll cost features.
    """

    ref: str | Unset = UNSET
    type_: str | Unset = UNSET
    name: str | Unset = UNSET
    category: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        ref = self.ref

        type_ = self.type_

        name = self.name

        category = self.category

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if ref is not UNSET:
            field_dict["ref"] = ref
        if type_ is not UNSET:
            field_dict["type"] = type_
        if name is not UNSET:
            field_dict["name"] = name
        if category is not UNSET:
            field_dict["category"] = category

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        ref = d.pop("ref", UNSET)

        type_ = d.pop("type", UNSET)

        name = d.pop("name", UNSET)

        category = d.pop("category", UNSET)

        routing_zone = cls(
            ref=ref,
            type_=type_,
            name=name,
            category=category,
        )

        routing_zone.additional_properties = d
        return routing_zone

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.violated_charging_station_opening_hours_type import (
    ViolatedChargingStationOpeningHoursType,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="ViolatedChargingStationOpeningHours")


@_attrs_define
class ViolatedChargingStationOpeningHours:
    """A charging stop was planned at the destination of this section, but the `postActions` would not be completed within
    the opening hours.

        Attributes:
            type_ (ViolatedChargingStationOpeningHoursType): Detail type. Each type of detail might contain extra
                attributes.

                **NOTE:** The list of possible detail types may be extended in the future.
                The client application is expected to handle such a case gracefully.
            title (str | Unset): Detail title
            cause (str | Unset): Cause of the notice
            opening_hours (str | Unset): Specifies date and time period during which the restriction applies. Value is a
                string in the Time
                Domain format. Time Domain is part of the GDF (Geographic Data Files) specification, which is an ISO standard.
                Current standard is GDF 5.1 which is [ISO 20524-1:2020](https://www.iso.org/standard/68244.html).

                For a detailed description of the Time Domain specification and usage in routing services, please refer to
                the documentation available in the [Time Domain](https://www.here.com/docs/bundle/routing-api-developer-
                guide-v8/page/concepts/time-domain.html) page of the Developer Guide.
                 Example: -(d1){w1}(d3){d1}.
    """

    type_: ViolatedChargingStationOpeningHoursType
    title: str | Unset = UNSET
    cause: str | Unset = UNSET
    opening_hours: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        title = self.title

        cause = self.cause

        opening_hours = self.opening_hours

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
        })
        if title is not UNSET:
            field_dict["title"] = title
        if cause is not UNSET:
            field_dict["cause"] = cause
        if opening_hours is not UNSET:
            field_dict["opening_hours"] = opening_hours

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = ViolatedChargingStationOpeningHoursType(d.pop("type"))

        title = d.pop("title", UNSET)

        cause = d.pop("cause", UNSET)

        opening_hours = d.pop("opening_hours", UNSET)

        violated_charging_station_opening_hours = cls(
            type_=type_,
            title=title,
            cause=cause,
            opening_hours=opening_hours,
        )

        violated_charging_station_opening_hours.additional_properties = d
        return violated_charging_station_opening_hours

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="EncodedCorridorArea")


@_attrs_define
class EncodedCorridorArea:
    """A polygon defined as a [Flexible Polyline](https://github.com/heremaps/flexible-polyline) encoded string
    and with a specified radius that defines an area.

        Attributes:
            type_ (str):  Example: encodedCorridor.
            polyline (str): [Flexible Polyline](https://github.com/heremaps/flexible-polyline) that defines a supporting
                structure for a corridor.
                Notes:
                * Support only 2D polyline (without `elevation` specified).
                * Minimum count of vertices in polyline is 2.
                * Maximum count of vertices in polyline is 100.
                 Example: BFoz5xJ67i1B1B7PzIhaxL7Y.
            radius (int): The width in meters of the corridor determines the outline of the polygon.
    """

    type_: str
    polyline: str
    radius: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        polyline = self.polyline

        radius = self.radius

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "polyline": polyline,
            "radius": radius,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = d.pop("type")

        polyline = d.pop("polyline")

        radius = d.pop("radius")

        encoded_corridor_area = cls(
            type_=type_,
            polyline=polyline,
            radius=radius,
        )

        encoded_corridor_area.additional_properties = d
        return encoded_corridor_area

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.contact import Contact


T = TypeVar("T", bound="ContactInformation")


@_attrs_define
class ContactInformation:
    """
    Attributes:
        phone (list[Contact] | Unset):
        mobile (list[Contact] | Unset):
        toll_free (list[Contact] | Unset):
        fax (list[Contact] | Unset):
        www (list[Contact] | Unset):
        email (list[Contact] | Unset):
    """

    phone: list[Contact] | Unset = UNSET
    mobile: list[Contact] | Unset = UNSET
    toll_free: list[Contact] | Unset = UNSET
    fax: list[Contact] | Unset = UNSET
    www: list[Contact] | Unset = UNSET
    email: list[Contact] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        phone: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.phone, Unset):
            phone = []
            for phone_item_data in self.phone:
                phone_item = phone_item_data.to_dict()
                phone.append(phone_item)

        mobile: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.mobile, Unset):
            mobile = []
            for mobile_item_data in self.mobile:
                mobile_item = mobile_item_data.to_dict()
                mobile.append(mobile_item)

        toll_free: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.toll_free, Unset):
            toll_free = []
            for toll_free_item_data in self.toll_free:
                toll_free_item = toll_free_item_data.to_dict()
                toll_free.append(toll_free_item)

        fax: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.fax, Unset):
            fax = []
            for fax_item_data in self.fax:
                fax_item = fax_item_data.to_dict()
                fax.append(fax_item)

        www: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.www, Unset):
            www = []
            for www_item_data in self.www:
                www_item = www_item_data.to_dict()
                www.append(www_item)

        email: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.email, Unset):
            email = []
            for email_item_data in self.email:
                email_item = email_item_data.to_dict()
                email.append(email_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if phone is not UNSET:
            field_dict["phone"] = phone
        if mobile is not UNSET:
            field_dict["mobile"] = mobile
        if toll_free is not UNSET:
            field_dict["tollFree"] = toll_free
        if fax is not UNSET:
            field_dict["fax"] = fax
        if www is not UNSET:
            field_dict["www"] = www
        if email is not UNSET:
            field_dict["email"] = email

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.contact import Contact

        d = dict(src_dict)
        _phone = d.pop("phone", UNSET)
        phone: list[Contact] | Unset = UNSET
        if _phone is not UNSET:
            phone = []
            for phone_item_data in _phone:
                phone_item = Contact.from_dict(phone_item_data)

                phone.append(phone_item)

        _mobile = d.pop("mobile", UNSET)
        mobile: list[Contact] | Unset = UNSET
        if _mobile is not UNSET:
            mobile = []
            for mobile_item_data in _mobile:
                mobile_item = Contact.from_dict(mobile_item_data)

                mobile.append(mobile_item)

        _toll_free = d.pop("tollFree", UNSET)
        toll_free: list[Contact] | Unset = UNSET
        if _toll_free is not UNSET:
            toll_free = []
            for toll_free_item_data in _toll_free:
                toll_free_item = Contact.from_dict(toll_free_item_data)

                toll_free.append(toll_free_item)

        _fax = d.pop("fax", UNSET)
        fax: list[Contact] | Unset = UNSET
        if _fax is not UNSET:
            fax = []
            for fax_item_data in _fax:
                fax_item = Contact.from_dict(fax_item_data)

                fax.append(fax_item)

        _www = d.pop("www", UNSET)
        www: list[Contact] | Unset = UNSET
        if _www is not UNSET:
            www = []
            for www_item_data in _www:
                www_item = Contact.from_dict(www_item_data)

                www.append(www_item)

        _email = d.pop("email", UNSET)
        email: list[Contact] | Unset = UNSET
        if _email is not UNSET:
            email = []
            for email_item_data in _email:
                email_item = Contact.from_dict(email_item_data)

                email.append(email_item)

        contact_information = cls(
            phone=phone,
            mobile=mobile,
            toll_free=toll_free,
            fax=fax,
            www=www,
            email=email,
        )

        contact_information.additional_properties = d
        return contact_information

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="MatchTraceVia")


@_attrs_define
class MatchTraceVia:
    """Via waypoint in the middle of route

    Attributes:
        index (int): Index of the corresponding trace point
        stop_duration (int | Unset): Duration in seconds. Example: 198.
    """

    index: int
    stop_duration: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        index = self.index

        stop_duration = self.stop_duration

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "index": index,
        })
        if stop_duration is not UNSET:
            field_dict["stopDuration"] = stop_duration

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        index = d.pop("index")

        stop_duration = d.pop("stopDuration", UNSET)

        match_trace_via = cls(
            index=index,
            stop_duration=stop_duration,
        )

        match_trace_via.additional_properties = d
        return match_trace_via

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.traffic_mode import TrafficMode
from ..types import UNSET, Unset

T = TypeVar("T", bound="Traffic")


@_attrs_define
class Traffic:
    """Traffic specific parameters.

    Attributes:
        override_flow_duration (int | Unset): Duration in seconds for which flow traffic event would be considered
            valid. While flow
            traffic event is valid it will be used over the historical traffic data.

            **Note**: Flow traffic represents congestion not caused by any long-term incidents.
            State of the flow traffic often changes fast. The farther away from the current time we
            move, the less precise current flow traffic data will be and the more precise historical
            traffic data becomes. That's why it's advised not to use this parameter unless you know
            what you want to achieve and use the default behavior which is almost always better.
        mode (TrafficMode | Unset): Defines what traffic data should be used for route shape and travel duration
            calculation.

            * `default`: Traffic data is considered.
            *    - If `departureTime=any` then only long-term closures will be considered.
            *    - If `departureTime` is not equal to `any` then all traffic data will be taken into account.
            * `disabled`: All traffic data, including long term closures, is ignored.
             Default: TrafficMode.DEFAULT.
    """

    override_flow_duration: int | Unset = UNSET
    mode: TrafficMode | Unset = TrafficMode.DEFAULT
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        override_flow_duration = self.override_flow_duration

        mode: str | Unset = UNSET
        if not isinstance(self.mode, Unset):
            mode = self.mode.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if override_flow_duration is not UNSET:
            field_dict["overrideFlowDuration"] = override_flow_duration
        if mode is not UNSET:
            field_dict["mode"] = mode

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        override_flow_duration = d.pop("overrideFlowDuration", UNSET)

        _mode = d.pop("mode", UNSET)
        mode: TrafficMode | Unset
        if isinstance(_mode, Unset):
            mode = UNSET
        else:
            mode = TrafficMode(_mode)

        traffic = cls(
            override_flow_duration=override_flow_duration,
            mode=mode,
        )

        traffic.additional_properties = d
        return traffic

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Exclude")


@_attrs_define
class Exclude:
    """Options to exclude strictly during the route calculation.

    Attributes:
        countries (str | Unset): A comma separated list of three-letter country codes (ISO-3166-1 alpha-3 code) that
            routes will exclude.
            - Format: `{country1}[,country2][,country...]`
              + `{country}` - Country code according to (ISO-3166-1 alpha-3)
            - Example: `CZE,SVK` - exclude Czechia and Slovakia from routing.

            Note - Exclude countries guarantees exclusion, but doesn't guarantee finding a route.
        states (str | Unset): A pipe separated list of country-aggregated state codes.

            - Format: `{country1}:{state1.1}[,state1.2][,state1.3]...[|{country2}:{state2.1}[,state2.2]]...`
              + `{country}` - Country code according to (ISO-3166-1 alpha-3)
              + `{state}`   - State code part according to (ISO 3166-2)
            - Examples:
              + `USA:NY,PA`       - exclude USA's New York and Pennsylvania states
              + `USA:NY,PA|CAN:QC`- exclude USA's New York, Pennsylvania states and Canada's Quebec province
            - Notes:
              + Exclude states guarantees exclusion, but doesn't guarantee finding a route.
              + Some countries don't have the state code data available for them, so the states in those countries won't be
            excluded.
        areas (str | Unset): A pipe separated list of user-defined areas that routes should avoid/exclude going through.

            Notes:
            * Maximum count of avoided and excluded polygons and corridors is 20.
            * Maximum total count of avoided and excluded bounding boxes, polygons, corridors, including exceptions, is 250.

            Format: `{area1}[!exception1.1[!exception1.2...]]|{area2}[!exception2.1[!exception2.2...]]|{area3}[!exception3.1
            [!exception3.2...]]...`

            Supported areas:
            * Bounding box - A rectangular area on earth defined by a comma separated list of two latitude and two longitude
            values.
              - Format: `bbox:{west},{south},{east},{north}`
                + `{west}`  - Longitude value of the westernmost point of the area.
                + `{south}` - Latitude value of the southernmost point of the area.
                + `{east}`  - Longitude value of the easternmost point of the area.
                + `{north}` - Latitude value of the northernmost point of the area.
              - Example: `bbox:13.082,52.416,13.628,52.626` - Bounding box of Berlin
            * Polygon - A polygon on earth which defines area.
              Possible formats:
              1) As list of geopoints.
                - Format: `polygon:{lat},{lon};{lat},{lon};{lat},{lon}...`
                  + `{lat}` - Latitude
                  + `{lon}` - Longitude
                - Example: `polygon:52.416,13.082;52.626,13.628;52.916,13.482` - Polygon in Berlin
              2) As [Flexible Polyline](https://github.com/heremaps/flexible-polyline) Encoding.
                - Support only 2D polyline (without `elevation` specified).
                - Format: `polygon:{encoded_polyline}`
                  + `{encoded_polyline}` - encoded [Flexible Polyline](https://github.com/heremaps/flexible-polyline)
                - Example: `polygon:BF05xgKuy2xCx9B7vUl0OhnR54EqSzpEl-HxjD3pBiGnyGi2CvwFsgD3nD4vB6e`
            * Corridor - A polyline with a specified radius (integer, in meters) that defines width of corridor area.
              Possible formats:
              1) As a list of geopoints that defines a polyline and a radius that defines an area around the polyline.
                - Format: `corridor:{lat},{lon};{lat},{lon};{lat},{lon}...;r={radius}`
                - Example: `corridor:52.416,13.082;52.626,13.628;52.916,13.482;r=1000`
              2) As [Flexible Polyline](https://github.com/heremaps/flexible-polyline) encoding and a radius that defines an
            area around the polyline.
                - Supports only 2D polyline (without `elevation` specified).
                - Format: `corridor:{encoded_polyline};r={radius}`
                  + `{encoded_polyline}` - encoded [Flexible Polyline](https://github.com/heremaps/flexible-polyline)
                - Example: `corridor:BF05xgKuy2xCx9B7vUl0OhnR54EqSzpEl-HxjD3pBiGnyGi2CvwFsgD3nD4vB6e;r=1000`

            * Exception - area to exclude from avoidance. Any area type can be specified. Any area type can be used as an
            exception. Multiple exceptions can be specified for each area.
              - Format: `{area to avoid/exclude}!exception={area}`
              - Example: `bbox:13.082,52.416,13.628,52.626!exception=polygon:BF05xgKuy2xCx9B7vUl0OhnR54EqSzpEl-
            HxjD3pBiGnyGi2CvwFsgD3nD4vB6e!exception=bbox:13.082,52.416,13.628,52.626`

              Notes:
              * Maximum count of avoided and excluded polygons and corridors is 20.
              * Maximum total count of avoided and excluded bounding boxes, polygons, corridors, including exceptions, is
            250.
              * Minimum count of coordinates in one polygon is 3
              * Minimum count of coordinates in any single corridor is 2.
              * Maximum count of coordinates in one polygon or in one corridor is 100
              * The polygon is closed automatically, there is no need to duplicate the first point as the last one.
              * Self-intersecting polygons are not supported.
    """

    countries: str | Unset = UNSET
    states: str | Unset = UNSET
    areas: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        countries = self.countries

        states = self.states

        areas = self.areas

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if countries is not UNSET:
            field_dict["countries"] = countries
        if states is not UNSET:
            field_dict["states"] = states
        if areas is not UNSET:
            field_dict["areas"] = areas

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        countries = d.pop("countries", UNSET)

        states = d.pop("states", UNSET)

        areas = d.pop("areas", UNSET)

        exclude = cls(
            countries=countries,
            states=states,
            areas=areas,
        )

        exclude.additional_properties = d
        return exclude

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="LicensePlateRestriction")


@_attrs_define
class LicensePlateRestriction:
    """Contains details of the violated license plate restriction.

    Attributes:
        type_ (str | Unset): Extensible enum: `lastCharacter` `...`
            Specifies the components of the vehicle license plate
            considered for the restriction.
        forbidden_characters (list[str] | Unset): A list of restricted characters in the vehicle license plate.

            The `type` property indicates which character of the vehicle license plate is considered for matching.
            The condition is met if that character matches any of the characters in the `forbiddenCharacters` array.

            For example, if `type` is set to `lastCharacter` and `forbiddenCharacters` is set to `["7", "8"]` then the
            condition is met if the last character
            of the vehicle license plate is either 7 or 8.
             Example: ['1', '3', '5', '7', '9'].
    """

    type_: str | Unset = UNSET
    forbidden_characters: list[str] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        forbidden_characters: list[str] | Unset = UNSET
        if not isinstance(self.forbidden_characters, Unset):
            forbidden_characters = self.forbidden_characters

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type_ is not UNSET:
            field_dict["type"] = type_
        if forbidden_characters is not UNSET:
            field_dict["forbiddenCharacters"] = forbidden_characters

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = d.pop("type", UNSET)

        forbidden_characters = cast(list[str], d.pop("forbiddenCharacters", UNSET))

        license_plate_restriction = cls(
            type_=type_,
            forbidden_characters=forbidden_characters,
        )

        license_plate_restriction.additional_properties = d
        return license_plate_restriction

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.category_ref import CategoryRef


T = TypeVar("T", bound="Contact")


@_attrs_define
class Contact:
    """
    Attributes:
        value (str): Contact information, as specified by the contact type.
        label (str | Unset): Optional label for the contact string, such as "Customer Service" or "Pharmacy Fax".
        categories (list[CategoryRef] | Unset): The list of place categories this contact refers to.
    """

    value: str
    label: str | Unset = UNSET
    categories: list[CategoryRef] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        value = self.value

        label = self.label

        categories: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.categories, Unset):
            categories = []
            for categories_item_data in self.categories:
                categories_item = categories_item_data.to_dict()
                categories.append(categories_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "value": value,
        })
        if label is not UNSET:
            field_dict["label"] = label
        if categories is not UNSET:
            field_dict["categories"] = categories

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.category_ref import CategoryRef

        d = dict(src_dict)
        value = d.pop("value")

        label = d.pop("label", UNSET)

        _categories = d.pop("categories", UNSET)
        categories: list[CategoryRef] | Unset = UNSET
        if _categories is not UNSET:
            categories = []
            for categories_item_data in _categories:
                categories_item = CategoryRef.from_dict(categories_item_data)

                categories.append(categories_item)

        contact = cls(
            value=value,
            label=label,
            categories=categories,
        )

        contact.additional_properties = d
        return contact

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GetRevgeocodeShowMapReferencesItem(str, Enum):
    ADMINIDS = "adminIds"
    CMVERSION = "cmVersion"
    LINKS = "links"
    MICROPOINTADDRESS = "microPointAddress"
    POINTADDRESS = "pointAddress"
    SEGMENTS = "segments"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.charging_station_place_side_of_street import (
    ChargingStationPlaceSideOfStreet,
)
from ..models.charging_station_place_type import ChargingStationPlaceType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.charge_point_operator import ChargePointOperator
    from ..models.charging_connector_attributes import ChargingConnectorAttributes
    from ..models.charging_station_brand import ChargingStationBrand
    from ..models.e_mobility_service_provider import EMobilityServiceProvider
    from ..models.location import Location


T = TypeVar("T", bound="ChargingStationPlace")


@_attrs_define
class ChargingStationPlace:
    """A charging station

    Example:
        {'type': 'place', 'location': {'lat': 50.339167, 'lng': 18.93}}

    Attributes:
        type_ (ChargingStationPlaceType): Place type. Each place type can have extra attributes.

            **NOTE:** The list of possible place types could be extended in the future.
            The client application is expected to handle such a case gracefully.
        location (Location): Location on the Earth Example: {'lat': 52.531677, 'lng': 13.381777}.
        name (str | Unset): Human readable name of this charging station
        waypoint (int | Unset): If present, this place corresponds to the `via` in the request with the same index.

            Example:

            If the request contains `via=<a>&via=<b>`
            * the place corresponding to `<a>` in the response will have `waypoint: 0`
            * the place corresponding to `<b>` in the response will have `waypoint: 1`.

            Notes:
            * `waypoint` is not present for `origin` and `destination` places. `origin` and `destination` waypoints can be
            trivially identified in the response as the `departure` of the first section and the `arrival` of the final
            section, respectively.
            * `waypoint` is not present for any stops that are added automatically by the router.
        original_location (Location | Unset): Location on the Earth Example: {'lat': 52.531677, 'lng': 13.381777}.
        display_location (Location | Unset): Location on the Earth Example: {'lat': 52.531677, 'lng': 13.381777}.
        side_of_street (ChargingStationPlaceSideOfStreet | Unset): Location of the waypoint or destination, relative to
            the driving direction on the street/road.

            **NOTE:** Based on the original waypoint position and `sideOfStreetHint` waypoint property.
             - If `sideOfStreetHint` property is specified, it takes priority in determining the side of street in the
            response.
             - Nothing is returned in the response in case the side of street is ambiguous (too close to the street),
               or if the position is too far from the street.

            * 'left`: The left side of the street in relation to the driving direction of the route.
            * `right`: The right side of the street in relation to the driving direction of the route.
        id (str | Unset): Identifier of this charging station
        attributes (ChargingConnectorAttributes | Unset): Details of the connector that is suggested to be used in the
            section's `postAction` for charging.
        brand (ChargingStationBrand | Unset): Information regarding the charging station brand
        charge_point_operator (ChargePointOperator | Unset): Information about the charging station charge-point-
            operator
        matching_e_mobility_service_providers (list[EMobilityServiceProvider] | Unset): List of matched E-Mobility
            Service Providers.
            Populated only when `ev[eMobilityServiceProviderPreferences]` parameter was
            passed in query.

            **Note**:
            * This list reflects the subset of E-Mobility Service Providers supported by the charging station, from the list
            specified in the request parameter `ev[eMobilityServiceProviderPreferences]`.
    """

    type_: ChargingStationPlaceType
    location: Location
    name: str | Unset = UNSET
    waypoint: int | Unset = UNSET
    original_location: Location | Unset = UNSET
    display_location: Location | Unset = UNSET
    side_of_street: ChargingStationPlaceSideOfStreet | Unset = UNSET
    id: str | Unset = UNSET
    attributes: ChargingConnectorAttributes | Unset = UNSET
    brand: ChargingStationBrand | Unset = UNSET
    charge_point_operator: ChargePointOperator | Unset = UNSET
    matching_e_mobility_service_providers: list[EMobilityServiceProvider] | Unset = (
        UNSET
    )
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        location = self.location.to_dict()

        name = self.name

        waypoint = self.waypoint

        original_location: dict[str, Any] | Unset = UNSET
        if not isinstance(self.original_location, Unset):
            original_location = self.original_location.to_dict()

        display_location: dict[str, Any] | Unset = UNSET
        if not isinstance(self.display_location, Unset):
            display_location = self.display_location.to_dict()

        side_of_street: str | Unset = UNSET
        if not isinstance(self.side_of_street, Unset):
            side_of_street = self.side_of_street.value

        id = self.id

        attributes: dict[str, Any] | Unset = UNSET
        if not isinstance(self.attributes, Unset):
            attributes = self.attributes.to_dict()

        brand: dict[str, Any] | Unset = UNSET
        if not isinstance(self.brand, Unset):
            brand = self.brand.to_dict()

        charge_point_operator: dict[str, Any] | Unset = UNSET
        if not isinstance(self.charge_point_operator, Unset):
            charge_point_operator = self.charge_point_operator.to_dict()

        matching_e_mobility_service_providers: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.matching_e_mobility_service_providers, Unset):
            matching_e_mobility_service_providers = []
            for (
                matching_e_mobility_service_providers_item_data
            ) in self.matching_e_mobility_service_providers:
                matching_e_mobility_service_providers_item = (
                    matching_e_mobility_service_providers_item_data.to_dict()
                )
                matching_e_mobility_service_providers.append(
                    matching_e_mobility_service_providers_item
                )

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "location": location,
        })
        if name is not UNSET:
            field_dict["name"] = name
        if waypoint is not UNSET:
            field_dict["waypoint"] = waypoint
        if original_location is not UNSET:
            field_dict["originalLocation"] = original_location
        if display_location is not UNSET:
            field_dict["displayLocation"] = display_location
        if side_of_street is not UNSET:
            field_dict["sideOfStreet"] = side_of_street
        if id is not UNSET:
            field_dict["id"] = id
        if attributes is not UNSET:
            field_dict["attributes"] = attributes
        if brand is not UNSET:
            field_dict["brand"] = brand
        if charge_point_operator is not UNSET:
            field_dict["chargePointOperator"] = charge_point_operator
        if matching_e_mobility_service_providers is not UNSET:
            field_dict["matchingEMobilityServiceProviders"] = (
                matching_e_mobility_service_providers
            )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.charge_point_operator import ChargePointOperator
        from ..models.charging_connector_attributes import ChargingConnectorAttributes
        from ..models.charging_station_brand import ChargingStationBrand
        from ..models.e_mobility_service_provider import EMobilityServiceProvider
        from ..models.location import Location

        d = dict(src_dict)
        type_ = ChargingStationPlaceType(d.pop("type"))

        location = Location.from_dict(d.pop("location"))

        name = d.pop("name", UNSET)

        waypoint = d.pop("waypoint", UNSET)

        _original_location = d.pop("originalLocation", UNSET)
        original_location: Location | Unset
        if isinstance(_original_location, Unset):
            original_location = UNSET
        else:
            original_location = Location.from_dict(_original_location)

        _display_location = d.pop("displayLocation", UNSET)
        display_location: Location | Unset
        if isinstance(_display_location, Unset):
            display_location = UNSET
        else:
            display_location = Location.from_dict(_display_location)

        _side_of_street = d.pop("sideOfStreet", UNSET)
        side_of_street: ChargingStationPlaceSideOfStreet | Unset
        if isinstance(_side_of_street, Unset):
            side_of_street = UNSET
        else:
            side_of_street = ChargingStationPlaceSideOfStreet(_side_of_street)

        id = d.pop("id", UNSET)

        _attributes = d.pop("attributes", UNSET)
        attributes: ChargingConnectorAttributes | Unset
        if isinstance(_attributes, Unset):
            attributes = UNSET
        else:
            attributes = ChargingConnectorAttributes.from_dict(_attributes)

        _brand = d.pop("brand", UNSET)
        brand: ChargingStationBrand | Unset
        if isinstance(_brand, Unset):
            brand = UNSET
        else:
            brand = ChargingStationBrand.from_dict(_brand)

        _charge_point_operator = d.pop("chargePointOperator", UNSET)
        charge_point_operator: ChargePointOperator | Unset
        if isinstance(_charge_point_operator, Unset):
            charge_point_operator = UNSET
        else:
            charge_point_operator = ChargePointOperator.from_dict(
                _charge_point_operator
            )

        _matching_e_mobility_service_providers = d.pop(
            "matchingEMobilityServiceProviders", UNSET
        )
        matching_e_mobility_service_providers: (
            list[EMobilityServiceProvider] | Unset
        ) = UNSET
        if _matching_e_mobility_service_providers is not UNSET:
            matching_e_mobility_service_providers = []
            for (
                matching_e_mobility_service_providers_item_data
            ) in _matching_e_mobility_service_providers:
                matching_e_mobility_service_providers_item = (
                    EMobilityServiceProvider.from_dict(
                        matching_e_mobility_service_providers_item_data
                    )
                )

                matching_e_mobility_service_providers.append(
                    matching_e_mobility_service_providers_item
                )

        charging_station_place = cls(
            type_=type_,
            location=location,
            name=name,
            waypoint=waypoint,
            original_location=original_location,
            display_location=display_location,
            side_of_street=side_of_street,
            id=id,
            attributes=attributes,
            brand=brand,
            charge_point_operator=charge_point_operator,
            matching_e_mobility_service_providers=matching_e_mobility_service_providers,
        )

        charging_station_place.additional_properties = d
        return charging_station_place

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.admin_names_preference import AdminNamesPreference
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.name import Name


T = TypeVar("T", bound="AdminNames")


@_attrs_define
class AdminNames:
    """
    Attributes:
        names (list[Name]): The list of all values for a name. Those might be for different languages or different name
            types.
        preference (AdminNamesPreference | Unset): The preference of the view on an address field expressed by this
            group of names.

            Description of supported values:

            - `alternative`: Alternative view on this address field. Names of this group are exposed in the result only if
            matching to the query so that the end-user can better recognize the result.
            - `primary`: The default or only view on this address filed. These are the names which are exposed in the result
            by default.
    """

    names: list[Name]
    preference: AdminNamesPreference | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        names = []
        for names_item_data in self.names:
            names_item = names_item_data.to_dict()
            names.append(names_item)

        preference: str | Unset = UNSET
        if not isinstance(self.preference, Unset):
            preference = self.preference.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "names": names,
        })
        if preference is not UNSET:
            field_dict["preference"] = preference

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.name import Name

        d = dict(src_dict)
        names = []
        _names = d.pop("names")
        for names_item_data in _names:
            names_item = Name.from_dict(names_item_data)

            names.append(names_item)

        _preference = d.pop("preference", UNSET)
        preference: AdminNamesPreference | Unset
        if isinstance(_preference, Unset):
            preference = UNSET
        else:
            preference = AdminNamesPreference(_preference)

        admin_names = cls(
            names=names,
            preference=preference,
        )

        admin_names.additional_properties = d
        return admin_names

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.tripadvisor_media_supplier import TripadvisorMediaSupplier


T = TypeVar("T", bound="Rating")


@_attrs_define
class Rating:
    """
    Attributes:
        count (int): The current count of user reviews in the supplier database.
        average (float): The current average of user ratings in the supplier database.
        supplier (TripadvisorMediaSupplier):
        href (str | Unset): An optional deep link to a 3rd party source providing the ratings information.
    """

    count: int
    average: float
    supplier: TripadvisorMediaSupplier
    href: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        count = self.count

        average = self.average

        supplier = self.supplier.to_dict()

        href = self.href

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "count": count,
            "average": average,
            "supplier": supplier,
        })
        if href is not UNSET:
            field_dict["href"] = href

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.tripadvisor_media_supplier import TripadvisorMediaSupplier

        d = dict(src_dict)
        count = d.pop("count")

        average = d.pop("average")

        supplier = TripadvisorMediaSupplier.from_dict(d.pop("supplier"))

        href = d.pop("href", UNSET)

        rating = cls(
            count=count,
            average=average,
            supplier=supplier,
            href=href,
        )

        rating.additional_properties = d
        return rating

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.toll_summary import TollSummary


T = TypeVar("T", bound="VehicleSummary")


@_attrs_define
class VehicleSummary:
    """Total value of key attributes for a route section.

    Attributes:
        duration (int): Duration in seconds. Example: 198.
        length (int): Distance in meters. Example: 189.
        base_duration (int | Unset): Duration in seconds. Example: 198.
        consumption (float | Unset): Energy or fuel consumption.
            For EV energy consumption is in kilowatt hours (kWh). For fuel-based vehicles fuel consumption is in Liters (L)
            for diesel, petrol and LPG vehicles, and Kilograms (kg) for CNG vehicles.
        typical_duration (int | Unset): Duration in seconds. Example: 198.
        ml_duration (int | Unset): Duration in seconds. Example: 198.
        tolls (TollSummary | Unset): The summary of the tolls grouped by criteria (total, per system, per country).
        co_2_emission (float | Unset): Estimation of the carbon dioxide emission for the given route section. Unit is
            kilograms with precision to three decimal places.

            Definitions:

            * Total Emission (Well-to-Wheel) = Production Emission (Well-to-Tank) + Operational Emission (Tank-to-Wheel)
            * CO2 Equivalent Emission (CO2e for CH4, N2O, etc.) = Gas Emission * Global Warming Potential (GWP)

            **Notes:**

            * This value represents the operational (tank-to-wheel) carbon dioxide emission.
            * This value represents only the carbon dioxide emission and does not represent the carbon dioxide equivalent
            emission for other gases or particulates.
    """

    duration: int
    length: int
    base_duration: int | Unset = UNSET
    consumption: float | Unset = UNSET
    typical_duration: int | Unset = UNSET
    ml_duration: int | Unset = UNSET
    tolls: TollSummary | Unset = UNSET
    co_2_emission: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        duration = self.duration

        length = self.length

        base_duration = self.base_duration

        consumption = self.consumption

        typical_duration = self.typical_duration

        ml_duration = self.ml_duration

        tolls: dict[str, Any] | Unset = UNSET
        if not isinstance(self.tolls, Unset):
            tolls = self.tolls.to_dict()

        co_2_emission = self.co_2_emission

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "duration": duration,
            "length": length,
        })
        if base_duration is not UNSET:
            field_dict["baseDuration"] = base_duration
        if consumption is not UNSET:
            field_dict["consumption"] = consumption
        if typical_duration is not UNSET:
            field_dict["typicalDuration"] = typical_duration
        if ml_duration is not UNSET:
            field_dict["mlDuration"] = ml_duration
        if tolls is not UNSET:
            field_dict["tolls"] = tolls
        if co_2_emission is not UNSET:
            field_dict["co2Emission"] = co_2_emission

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.toll_summary import TollSummary

        d = dict(src_dict)
        duration = d.pop("duration")

        length = d.pop("length")

        base_duration = d.pop("baseDuration", UNSET)

        consumption = d.pop("consumption", UNSET)

        typical_duration = d.pop("typicalDuration", UNSET)

        ml_duration = d.pop("mlDuration", UNSET)

        _tolls = d.pop("tolls", UNSET)
        tolls: TollSummary | Unset
        if isinstance(_tolls, Unset):
            tolls = UNSET
        else:
            tolls = TollSummary.from_dict(_tolls)

        co_2_emission = d.pop("co2Emission", UNSET)

        vehicle_summary = cls(
            duration=duration,
            length=length,
            base_duration=base_duration,
            consumption=consumption,
            typical_duration=typical_duration,
            ml_duration=ml_duration,
            tolls=tolls,
            co_2_emission=co_2_emission,
        )

        vehicle_summary.additional_properties = d
        return vehicle_summary

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CalculateIsolinesPostOptimizeFor(str, Enum):
    BALANCED = "balanced"
    PERFORMANCE = "performance"
    QUALITY = "quality"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="TimeZoneInfo")


@_attrs_define
class TimeZoneInfo:
    """
    Attributes:
        name (str): The name of the time zone as defined in the [tz
            database](https://en.wikipedia.org/wiki/Tz_database). For example: "Europe/Berlin"
        utc_offset (str): The UTC offset for this time zone at request time. For example "+02:00"
    """

    name: str
    utc_offset: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        utc_offset = self.utc_offset

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "name": name,
            "utcOffset": utc_offset,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        utc_offset = d.pop("utcOffset")

        time_zone_info = cls(
            name=name,
            utc_offset=utc_offset,
        )

        time_zone_info.additional_properties = d
        return time_zone_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.transit_departure import TransitDeparture
    from ..models.transit_transport import TransitTransport


T = TypeVar("T", bound="TransitStop")


@_attrs_define
class TransitStop:
    """A transit stop of the route

    Attributes:
        departure (TransitDeparture): Transit departure
        duration (int | Unset): Stop duration. If not set the vehicle departs as soon as people are on board. Default:
            0. Example: 198.
        transport (TransitTransport | Unset): Transit transport information.
        offset (int | Unset): The position of the stop on the polyline.
        attributes (list[str] | Unset): **NOTE:** As it is possible that new attributes are supported in the future,
            unknown attributes should be ignored.
    """

    departure: TransitDeparture
    duration: int | Unset = 0
    transport: TransitTransport | Unset = UNSET
    offset: int | Unset = UNSET
    attributes: list[str] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        departure = self.departure.to_dict()

        duration = self.duration

        transport: dict[str, Any] | Unset = UNSET
        if not isinstance(self.transport, Unset):
            transport = self.transport.to_dict()

        offset = self.offset

        attributes: list[str] | Unset = UNSET
        if not isinstance(self.attributes, Unset):
            attributes = self.attributes

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "departure": departure,
        })
        if duration is not UNSET:
            field_dict["duration"] = duration
        if transport is not UNSET:
            field_dict["transport"] = transport
        if offset is not UNSET:
            field_dict["offset"] = offset
        if attributes is not UNSET:
            field_dict["attributes"] = attributes

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.transit_departure import TransitDeparture
        from ..models.transit_transport import TransitTransport

        d = dict(src_dict)
        departure = TransitDeparture.from_dict(d.pop("departure"))

        duration = d.pop("duration", UNSET)

        _transport = d.pop("transport", UNSET)
        transport: TransitTransport | Unset
        if isinstance(_transport, Unset):
            transport = UNSET
        else:
            transport = TransitTransport.from_dict(_transport)

        offset = d.pop("offset", UNSET)

        attributes = cast(list[str], d.pop("attributes", UNSET))

        transit_stop = cls(
            departure=departure,
            duration=duration,
            transport=transport,
            offset=offset,
            attributes=attributes,
        )

        transit_stop.additional_properties = d
        return transit_stop

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.violated_zone_reference_type import ViolatedZoneReferenceType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.license_plate_restriction import LicensePlateRestriction
    from ..models.vehicle_restriction_max_weight import VehicleRestrictionMaxWeight


T = TypeVar("T", bound="ViolatedZoneReference")


@_attrs_define
class ViolatedZoneReference:
    r"""Provides details about the zone associated with `violatedZoneRestriction` and
    the violated restriction conditions that were evaluated against parameter values
    set in the routing request (such as vehicle properties).

    The restriction is violated only if all of the conditions present are met.

        Attributes:
            type_ (ViolatedZoneReferenceType): Detail type. Each type of detail might contain extra attributes.

                **NOTE:** The list of possible detail types may be extended in the future.
                The client application is expected to handle such a case gracefully.
            title (str | Unset): Detail title
            cause (str | Unset): Cause of the notice
            routing_zone_ref (str | Unset): A reference to a routing zone in HMC.

                The standard representation of a routing zone reference has the following structure:
                `{catalogHrn}:{catalogVersion}:({layerId})?:{tileId}:{zoneId}`

                The individual parts are:
                * `catalogHrn`: The HERE Resource Name that identifies the source catalog of the routing zone, example:
                `hrn:here:data::olp-here:rib-2`
                * `catalogVersion`: The catalog version
                * `layerId` (optional): The layer inside the catalog where the routing zone is located, example: `environmental-
                zones`
                * `tileId`: The HERE tile key of the partition/tile where the routing zone is located in the given version of
                the catalog
                * `zoneId`: The identifier of the referenced routing zone within the catalog, example: `here:cm:envzone:3455277`

                Example of a reference to an environmental zone in standard form:
                `hrn:here:data::olp-here:rib-2:1557:environmental-zones:all:here:cm:envzone:3455277`

                In order to reduce response size, routing zone references can also be provided in a compact representation.
                In compact form, parts of a reference are replaced by placeholders, which can be resolved using the
                `refReplacements` dictionary in the parent section.
                The placeholder format is ```\$\d+``` and needs to be surrounded by colons or string start/end. It can be
                captured with the following regular expression: ```(^|:)\$\d+(:|$)``` .

                Example of the aforementioned environmental zone reference in compact form: `$0:$1:3455277`
                With the corresponding `refReplacements`:
                ```
                "refReplacements": {
                  "0": "hrn:here:data::olp-here:rib-2:1557",
                  "1": "environmental-zones:all:here:cm:envzone"
                }
                ```
            time_dependent (bool | Unset): Indicates if the violation is time-dependent
            restricted_times (str | Unset): Specifies date and time period during which the restriction applies. Value is a
                string in the Time
                Domain format. Time Domain is part of the GDF (Geographic Data Files) specification, which is an ISO standard.
                Current standard is GDF 5.1 which is [ISO 20524-1:2020](https://www.iso.org/standard/68244.html).

                For a detailed description of the Time Domain specification and usage in routing services, please refer to
                the documentation available in the [Time Domain](https://www.here.com/docs/bundle/routing-api-developer-
                guide-v8/page/concepts/time-domain.html) page of the Developer Guide.
                 Example: -(d1){w1}(d3){d1}.
            license_plate_restriction (LicensePlateRestriction | Unset): Contains details of the violated license plate
                restriction.
            max_weight (VehicleRestrictionMaxWeight | Unset): Contains the maximum permitted weight, specified in kilograms,
                along with the specific type of the maximum permitted weight restriction.
    """

    type_: ViolatedZoneReferenceType
    title: str | Unset = UNSET
    cause: str | Unset = UNSET
    routing_zone_ref: str | Unset = UNSET
    time_dependent: bool | Unset = UNSET
    restricted_times: str | Unset = UNSET
    license_plate_restriction: LicensePlateRestriction | Unset = UNSET
    max_weight: VehicleRestrictionMaxWeight | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        title = self.title

        cause = self.cause

        routing_zone_ref = self.routing_zone_ref

        time_dependent = self.time_dependent

        restricted_times = self.restricted_times

        license_plate_restriction: dict[str, Any] | Unset = UNSET
        if not isinstance(self.license_plate_restriction, Unset):
            license_plate_restriction = self.license_plate_restriction.to_dict()

        max_weight: dict[str, Any] | Unset = UNSET
        if not isinstance(self.max_weight, Unset):
            max_weight = self.max_weight.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
        })
        if title is not UNSET:
            field_dict["title"] = title
        if cause is not UNSET:
            field_dict["cause"] = cause
        if routing_zone_ref is not UNSET:
            field_dict["routingZoneRef"] = routing_zone_ref
        if time_dependent is not UNSET:
            field_dict["timeDependent"] = time_dependent
        if restricted_times is not UNSET:
            field_dict["restrictedTimes"] = restricted_times
        if license_plate_restriction is not UNSET:
            field_dict["licensePlateRestriction"] = license_plate_restriction
        if max_weight is not UNSET:
            field_dict["maxWeight"] = max_weight

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.license_plate_restriction import LicensePlateRestriction
        from ..models.vehicle_restriction_max_weight import VehicleRestrictionMaxWeight

        d = dict(src_dict)
        type_ = ViolatedZoneReferenceType(d.pop("type"))

        title = d.pop("title", UNSET)

        cause = d.pop("cause", UNSET)

        routing_zone_ref = d.pop("routingZoneRef", UNSET)

        time_dependent = d.pop("timeDependent", UNSET)

        restricted_times = d.pop("restrictedTimes", UNSET)

        _license_plate_restriction = d.pop("licensePlateRestriction", UNSET)
        license_plate_restriction: LicensePlateRestriction | Unset
        if isinstance(_license_plate_restriction, Unset):
            license_plate_restriction = UNSET
        else:
            license_plate_restriction = LicensePlateRestriction.from_dict(
                _license_plate_restriction
            )

        _max_weight = d.pop("maxWeight", UNSET)
        max_weight: VehicleRestrictionMaxWeight | Unset
        if isinstance(_max_weight, Unset):
            max_weight = UNSET
        else:
            max_weight = VehicleRestrictionMaxWeight.from_dict(_max_weight)

        violated_zone_reference = cls(
            type_=type_,
            title=title,
            cause=cause,
            routing_zone_ref=routing_zone_ref,
            time_dependent=time_dependent,
            restricted_times=restricted_times,
            license_plate_restriction=license_plate_restriction,
            max_weight=max_weight,
        )

        violated_zone_reference.additional_properties = d
        return violated_zone_reference

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
